package org.bouml.plugout.apibase.com;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintStream;
import java.net.Socket;
import org.bouml.plugout.apibase.PlugoutException;
import org.bouml.plugout.apibase.type.aRelationKind;
import org.bouml.plugout.apibase.type.anItemKind;
import org.bouml.plugout.apibase.umlitem.UmlClass;
import org.bouml.plugout.apibase.umlitem.UmlItem;
import org.bouml.plugout.apibase.umlitem.UmlTypeSpec;

/**
 * This class manages the communications.
 * This class may be defined as a 'singleton', but I prefer to use static
 * members allowing to just write 'UmlCom::member' rather than
 * 'UmlCom::instance()->member' or other long sentence like this.
 * The operation you can use yourself are :
 * - connect()
 * - targetItem()
 * - trace()
 * - showTrace()
 * - traceAutoRaise()
 * - message()
 * - bye()
 * - close()
 * You must NOT call the others !
 */
public class UmlCom {
	private static byte[] buffer_in;

	private static int buffer_in_end;

	private static int buffer_in_size;

	private static byte[] buffer_out;

	private static int buffer_out_size;

	private static final String empty_string = new String();

	private static byte id_size;

	private static DataInputStream is;

	private static DataOutputStream os;

	private static int p_buffer_in;

	private static int p_buffer_out;

	private static Socket sock;

	/**
	 * Must be called just before the disconnexion.
	 */
	public static void bye() {
		send_cmd(CmdFamily.miscGlobalCmd, MiscGlobalCmd._byeCmd);
	}

	/**
	 * Internal, do NOT use it !
	 */
	public static void check() {
		if(!read_bool()) {
			throw new PlugoutException("Cannot be done");
		}
	}

	/**
	 * Internal, do NOT use it !
	 */
	protected static void check_size_out(int len) {
		if(p_buffer_out + len >= buffer_out_size) {
			buffer_out_size = p_buffer_out + len + 1024;
			final byte[] newbuff = new byte[buffer_out_size];
			int index;
			for(index = 0 ; index != p_buffer_out ; index += 1) {
				newbuff[index] = buffer_out[index];
			}
			buffer_out = newbuff;
		}
	}

	/**
	 * Disconnexion.
	 */
	public static void close() {
		try {
			os.close();
			is.close();
			sock.close();
			os = null;
			is = null;
			sock = null;
		} catch(final Exception exception) {
			throw new PlugoutException(exception.getMessage());
		}
	}

	/**
	 * Does the connexion.
	 * On error produce a PlugoutException.
	 */
	public static void connect(int port) throws PlugoutException {
		try {
			sock = new Socket("127.0.0.1", port);
			is = new DataInputStream(sock.getInputStream());
			os = new DataOutputStream(sock.getOutputStream());
			buffer_in_size = 1024;
			buffer_in = new byte[buffer_in_size];
			p_buffer_in = 0;
			buffer_out_size = 1024;
			buffer_out = new byte[buffer_out_size];
			p_buffer_out = 4; // bytes for length
			// Send API version.
			write_unsigned(48);
			flush();
		} catch(final Exception exception) {
			throw new PlugoutException("Cannot establish connexion : " + exception.getMessage());
		}
	}

	/**
	 * Internal, do NOT use it !
	 */
	public static void flush() {
		if(sock != null) {
			final int len = p_buffer_out - 4;
			/*
			 * the four first bytes of buffer_out are free to contains the
			 * length
			 */
			buffer_out[0] = (byte)(len >> 24);
			buffer_out[1] = (byte)(len >> 16);
			buffer_out[2] = (byte)(len >> 8);
			buffer_out[3] = (byte)len;
			try {
				os.write(buffer_out, 0, p_buffer_out);
				os.flush();
				p_buffer_out = 4; // bytes for length
			} catch(final Exception exception) {
				close(); // to not try to send "bye" !
				throw new PlugoutException(exception.getMessage());
			}
		}
	}

	/**
	 * To write a message in the status bar line does not use Qt rich text.
	 */
	public static void message(String s) {
		send_cmd(CmdFamily.miscGlobalCmd, MiscGlobalCmd._messageCmd, s);
	}

	/**
	 * Internal, do NOT use it !
	 */
	public static boolean read_bool() {
		read_if_needed();
		return buffer_in[p_buffer_in++] != 0;
	}

	/**
	 * Internal, do NOT use it !
	 */
	protected static void read_buffer(int len) {
		if(buffer_in_size < len) {
			buffer_in_size = len + 256;
			buffer_in = new byte[buffer_in_size];
		}
		int remainder = len;
		int offset = 0;
		int nread;
		for( ; ;) {
			try {
				if((nread = is.read(buffer_in, offset, remainder)) == -1) {
					nread = 0;
				}
			} catch(final Exception exception) {
				throw new PlugoutException("Communication closed : " + exception.getMessage());
			}
			if((remainder -= nread) == 0) {
				break;
			}
			offset += nread;
			// ...wait some time
		}
		p_buffer_in = 0;
		buffer_in_end = len;
	}

	/**
	 * Internal, do NOT use it !
	 */
	public static int read_char() {
		read_if_needed();
		return buffer_in[p_buffer_in++] & 255;
	}

	/**
	 * Internal, do NOT use it !
	 */
	public static long read_id() {
		read_if_needed();
		id_size = buffer_in[p_buffer_in++];
		long id;
		// id_size is supposed to be 8 or 4 !
		if(id_size == (byte)8) {
			id = (((long)buffer_in[p_buffer_in] & (long)255) << 56) + (((long)buffer_in[p_buffer_in + 1] & (long)255) << 48) + (((long)buffer_in[p_buffer_in + 2] & (long)255) << 40) + (((long)buffer_in[p_buffer_in + 3] & (long)255) << 32);
			p_buffer_in += 4;
		} else {
			id = 0;
		}
		id += (((long)buffer_in[p_buffer_in] & (long)255) << 24) + (((long)buffer_in[p_buffer_in + 1] & (long)255) << 16) + (((long)buffer_in[p_buffer_in + 2] & (long)255) << 8) + ((long)buffer_in[p_buffer_in + 3] & (long)255);
		p_buffer_in += 4;
		return id;
	}

	/**
	 * Internal, do NOT use it !
	 */
	protected static void read_if_needed() {
		if(p_buffer_in == buffer_in_end) {
			read_buffer(4);
			read_buffer(((buffer_in[0] & 255) << 24) + ((buffer_in[1] & 255) << 16) + ((buffer_in[2] & 255) << 8) + (buffer_in[3] & 255));
		}
	}

	/**
	 * Internal, do NOT use it !
	 */
	public static UmlItem[] read_item_list() {
		final int n = read_unsigned();
		final UmlItem[] v = new UmlItem[n];
		for(int index = 0 ; index != n ; index++) {
			v[index] = UmlItem.read_();
		}
		return v;
	}

	/**
	 * Internal, do NOT use it !
	 */
	public static String read_string() {
		read_if_needed();
		final int start = p_buffer_in;
		while(buffer_in[p_buffer_in++] != 0) {
			;
		}
		if(p_buffer_in == start + 1) {
			return empty_string;
		}
		return new String(buffer_in, start, p_buffer_in - start - 1);
	}

	/**
	 * Internal, do NOT use it !
	 */
	public static int read_unsigned() {
		read_if_needed();
		p_buffer_in += 4;
		return ((buffer_in[p_buffer_in - 4] & 255) << 24) + ((buffer_in[p_buffer_in - 3] & 255) << 16) + ((buffer_in[p_buffer_in - 2] & 255) << 8) + (buffer_in[p_buffer_in - 1] & 255);
	}

	/**
	 * Internal, do NOT use it !
	 */
	public static void send_cmd(CmdFamily f, int cmd) {
		write_char((byte)f.value());
		write_char((byte)cmd);
		flush();
	}

	/**
	 * Internal, do NOT use it !
	 */
	public static void send_cmd(CmdFamily f, int cmd, boolean b, String s, String v) {
		write_char((byte)f.value());
		write_char((byte)cmd);
		write_bool(b);
		write_string(s);
		write_string(v);
		flush();
	}

	/**
	 * Internal, do NOT use it !
	 */
	public static void send_cmd(CmdFamily f, int cmd, byte arg) {
		write_char((byte)f.value());
		write_char((byte)cmd);
		write_char(arg);
		flush();
	}

	/**
	 * Internal, do NOT use it !
	 */
	public static void send_cmd(CmdFamily f, int cmd, int arg, String dummy) {
		write_char((byte)f.value());
		write_char((byte)cmd);
		write_unsigned(arg);
		flush();
	}

	/**
	 * Internal, do NOT use it !
	 */
	public static void send_cmd(CmdFamily f, int cmd, long id) {
		write_char((byte)f.value());
		write_char((byte)cmd);
		write_id(id);
		flush();
	}

	/**
	 * Internal, do NOT use it !
	 */
	public static void send_cmd(CmdFamily f, int cmd, long id, String n) {
		write_char((byte)f.value());
		write_char((byte)cmd);
		write_id(id);
		write_string(n);
		flush();
	}

	/**
	 * Internal, do NOT use it !
	 */
	public static void send_cmd(CmdFamily f, int cmd, String s) {
		write_char((byte)f.value());
		write_char((byte)cmd);
		write_string(s);
		flush();
	}

	/**
	 * Internal, do NOT use it !
	 */
	public static void send_cmd(CmdFamily f, int cmd, String s, boolean b) {
		write_char((byte)f.value());
		write_char((byte)cmd);
		write_string(s);
		write_bool(b);
		flush();
	}

	/**
	 * Internal, do NOT use it !
	 */
	public static void send_cmd(CmdFamily f, int cmd, String s, String v) {
		write_char((byte)f.value());
		write_char((byte)cmd);
		write_string(s);
		write_string(v);
		flush();
	}

	/**
	 * Internal, do NOT use it !
	 */
	public static void send_cmd(long id, OnInstanceCmd cmd) {
		write_char((byte)CmdFamily._onInstanceCmd);
		write_id(id);
		write_char((byte)cmd.value());
		flush();
	}

	/**
	 * Internal, do NOT use it !
	 */
	public static void send_cmd(long id, OnInstanceCmd cmd, anItemKind arg, long id2) {
		write_char((byte)CmdFamily._onInstanceCmd);
		write_id(id);
		write_char((byte)cmd.value());
		write_char((byte)arg.value());
		write_id(id2);
		flush();
	}

	/**
	 * Internal, do NOT use it !
	 */
	public static void send_cmd(long id, OnInstanceCmd cmd, anItemKind arg1, String arg2) {
		write_char((byte)CmdFamily._onInstanceCmd);
		write_id(id);
		write_char((byte)cmd.value());
		write_char((byte)arg1.value());
		write_string(arg2);
		flush();
	}

	/**
	 * Internal, do NOT use it !
	 */
	public static void send_cmd(long id, OnInstanceCmd cmd, byte arg) {
		write_char((byte)CmdFamily._onInstanceCmd);
		write_id(id);
		write_char((byte)cmd.value());
		write_char(arg);
		flush();
	}

	/**
	 * Internal, do NOT use it !
	 */
	public static void send_cmd(long id, OnInstanceCmd cmd, int arg) {
		write_char((byte)CmdFamily._onInstanceCmd);
		write_id(id);
		write_char((byte)cmd.value());
		write_unsigned(arg);
		flush();
	}

	/**
	 * Internal, do NOT use it !
	 */
	public static void send_cmd(long id, OnInstanceCmd cmd, int arg1, byte arg2, String arg3, String arg4, UmlTypeSpec arg5) {
		write_char((byte)CmdFamily._onInstanceCmd);
		write_id(id);
		write_char((byte)cmd.value());
		write_unsigned(arg1);
		write_char(arg2);
		write_string(arg3);
		write_string(arg4);
		if(arg5.type != null) {
			write_id(arg5.type.identifier_());
			write_string(null);
		} else {
			write_id(0);
			write_string(arg5.explicit_type);
		}
		flush();
	}

	/**
	 * Internal, do NOT use it !
	 */
	public static void send_cmd(long id, OnInstanceCmd cmd, int arg1, String arg2, String arg3, UmlTypeSpec arg4, UmlTypeSpec arg5) {
		write_char((byte)CmdFamily._onInstanceCmd);
		write_id(id);
		write_char((byte)cmd.value());
		write_unsigned(arg1);
		write_string(arg2);
		write_string(arg3);
		if(arg4.type != null) {
			write_id(arg4.type.identifier_());
			write_string(null);
		} else {
			write_id(0);
			write_string(arg4.explicit_type);
		}
		if(arg5.type != null) {
			write_id(arg5.type.identifier_());
			write_string(null);
		} else {
			write_id(0);
			write_string(arg5.explicit_type);
		}
		flush();
	}

	/**
	 * Internal, do NOT use it !
	 */
	public static void send_cmd(long id, OnInstanceCmd cmd, int arg1, UmlTypeSpec arg2) {
		write_char((byte)CmdFamily._onInstanceCmd);
		write_id(id);
		write_char((byte)cmd.value());
		write_unsigned(arg1);
		if(arg2.type != null) {
			write_id(arg2.type.identifier_());
			write_string(null);
		} else {
			write_id(0);
			write_string(arg2.explicit_type);
		}
		flush();
	}

	/**
	 * Internal, do NOT use it !
	 */
	public static void send_cmd(long id, OnInstanceCmd cmd, long id1) {
		write_char((byte)CmdFamily._onInstanceCmd);
		write_id(id);
		write_char((byte)cmd.value());
		write_id(id1);
		flush();
	}

	/**
	 * Internal, do NOT use it !
	 */
	public static void send_cmd(long id, OnInstanceCmd cmd, long id1, String arg2) {
		write_char((byte)CmdFamily._onInstanceCmd);
		write_id(id);
		write_char((byte)cmd.value());
		write_id(id1);
		write_string(arg2);
		flush();
	}

	/**
	 * Internal, do NOT use it !
	 */
	public static void send_cmd(long id, OnInstanceCmd cmd, org.bouml.plugout.apibase.type.anItemKind arg1, aRelationKind arg2, long id2) {
		write_char((byte)CmdFamily._onInstanceCmd);
		write_id(id);
		write_char((byte)cmd.value());
		write_char((byte)arg1.value());
		write_char((byte)arg2.value());
		write_id(id2);
		flush();
	}

	/**
	 * Internal, do NOT use it !
	 */
	public static void send_cmd(long id, OnInstanceCmd cmd, String arg) {
		write_char((byte)CmdFamily._onInstanceCmd);
		write_id(id);
		write_char((byte)cmd.value());
		write_string(arg);
		flush();
	}

	/**
	 * Internal, do NOT use it !
	 */
	public static void send_cmd(long id, OnInstanceCmd cmd, String arg1, String arg2) {
		write_char((byte)CmdFamily._onInstanceCmd);
		write_id(id);
		write_char((byte)cmd.value());
		write_string(arg1);
		write_string(arg2);
		flush();
	}

	public static void send_cmd(long id, OnInstanceCmd cmd, UmlClass[] l1, UmlClass[] l2, UmlClass[] l3) {
		write_char((byte)CmdFamily._onInstanceCmd);
		write_id(id);
		write_char((byte)cmd.value());
		int n;
		n = l1.length;
		write_unsigned(n);
		for(int i = 0 ; i != n ; i++) {
			write_id(l1[i].identifier_());
		}
		n = l2.length;
		write_unsigned(n);
		for(int i = 0 ; i != n ; i++) {
			write_id(l2[i].identifier_());
		}
		n = l3.length;
		write_unsigned(n);
		for(int i = 0 ; i != n ; i++) {
			write_id(l3[i].identifier_());
		}
		flush();
	}

	/**
	 * Internal, do NOT use it !
	 */
	public static void send_cmd(long id, OnInstanceCmd cmd, UmlItem[] l) {
		write_char((byte)CmdFamily._onInstanceCmd);
		write_id(id);
		write_char((byte)cmd.value());
		final int n = l.length;
		write_unsigned(n);
		for(int i = 0 ; i != n ; i++) {
			write_id(l[i].identifier_());
		}
		flush();
	}

	/**
	 * Internal, do NOT use it !
	 */
	public static void send_cmd(long id, OnInstanceCmd cmd, UmlTypeSpec arg) {
		write_char((byte)CmdFamily._onInstanceCmd);
		write_id(id);
		write_char((byte)cmd.value());
		if(arg.type != null) {
			write_id(arg.type.identifier_());
			write_string(null);
		} else {
			write_id(0);
			write_string(arg.explicit_type);
		}
		flush();
	}

	/**
	 * To show the trace window.
	 */
	public static void showTrace() {
		send_cmd(CmdFamily.miscGlobalCmd, MiscGlobalCmd._showTraceCmd);
	}

	/**
	 * Returns the item on which the tool is applied.
	 */
	public static UmlItem targetItem() {
		send_cmd(CmdFamily.miscGlobalCmd, MiscGlobalCmd._targetCmd);
		return UmlItem.read_();
	}

	/**
	 * To write messages in the tool window, use Qt rich text : allows HTML like
	 * formatting.
	 */
	public static void trace(String s) {
		send_cmd(CmdFamily.miscGlobalCmd, MiscGlobalCmd._traceCmd, s);
	}

	/**
	 * To automatically raise the trace window each time trace() is called.
	 */
	public static void traceAutoRaise(boolean y) {
		send_cmd(CmdFamily.miscGlobalCmd, MiscGlobalCmd._traceAutoRaiseCmd, y ? (byte)1 : (byte)0);
	}

	public static void traceError(String error) {
		send_cmd(CmdFamily.miscGlobalCmd, MiscGlobalCmd._traceCmd, "<font color='red'>" + error + "</font>");
	}

	public static void traceException(Exception exception) {
		final StringBuilder buffer = new StringBuilder();
		final OutputStream outputStream = new OutputStream() {
			@Override
			public void write(int b) throws IOException {
				final char charToAppend = (char)b;
				buffer.append(charToAppend == '\n' ? "<br>" : charToAppend);
			}
		};
		final PrintStream printStream = new PrintStream(outputStream);
		exception.printStackTrace(new PrintStream(printStream));
		printStream.close();
		final String error = buffer.toString();
		traceError(error.substring(0, error.length() - 4));
	}

	/**
	 * Internal, do NOT use it !
	 */
	protected static void write_bool(boolean b) {
		check_size_out(1);
		buffer_out[p_buffer_out++] = b ? (byte)1 : (byte)0;
	}

	/**
	 * Internal, do NOT use it !
	 */
	protected static void write_char(byte c) {
		check_size_out(1);
		buffer_out[p_buffer_out++] = c;
	}

	/**
	 * Internal, do NOT use it !
	 */
	protected static void write_id(long id) {
		check_size_out(id_size);
		// id_size is supposed to be 8 or 4 !
		if(id_size == (byte)8) {
			buffer_out[p_buffer_out] = (byte)(id >> 56);
			buffer_out[p_buffer_out + 1] = (byte)(id >> 48);
			buffer_out[p_buffer_out + 2] = (byte)(id >> 40);
			buffer_out[p_buffer_out + 3] = (byte)(id >> 32);
			p_buffer_out += 4;
		}
		buffer_out[p_buffer_out] = (byte)(id >> 24);
		buffer_out[p_buffer_out + 1] = (byte)(id >> 16);
		buffer_out[p_buffer_out + 2] = (byte)(id >> 8);
		buffer_out[p_buffer_out + 3] = (byte)id;
		p_buffer_out += 4;
	}

	/**
	 * Internal, do NOT use it !
	 */
	protected static void write_string(String p) {
		if(p == null) {
			check_size_out(1);
			buffer_out[p_buffer_out++] = 0;
		} else {
			final byte[] pb = p.getBytes();
			int index;
			check_size_out(pb.length + 1);
			for(index = 0 ; index != pb.length ; index += 1) {
				buffer_out[p_buffer_out + index] = pb[index];
			}
			buffer_out[p_buffer_out + index] = 0;
			p_buffer_out += index + 1;
		}
	}

	/**
	 * Internal, do NOT use it !
	 */
	protected static void write_unsigned(int u) {
		check_size_out(4);
		buffer_out[p_buffer_out] = (byte)(u >> 24);
		buffer_out[p_buffer_out + 1] = (byte)(u >> 16);
		buffer_out[p_buffer_out + 2] = (byte)(u >> 8);
		buffer_out[p_buffer_out + 3] = (byte)u;
		p_buffer_out += 4;
	}
}
