package org.bouml.plugout.apibase.diagram;

import org.bouml.plugout.apibase.com.UmlCom;
import org.bouml.plugout.apibase.umlitem.UmlDiagram;
import org.bouml.plugout.apibase.umlitem.UmlItem;

/**
 * this class manages fragments
 */
public class UmlFragment {
	/**
	 * internal
	 */
	public static void compute_container_(UmlFragment[] fragments) {
		int rank = fragments.length;
		while(rank-- != 0) {
			final UmlFragment f = fragments[rank];
			final UmlFragmentCompartment fc = get_container_(f._x, f._y, f._w, f._h, fragments);
			if(fc != null) {
				f._container = fc;
				fc.add_contained_(f);
			}
		}
	}

	/**
	 * internal
	 */
	public static UmlFragmentCompartment get_container_(int x, int y, int w, int h, UmlFragment[] fragments) {
		UmlFragmentCompartment r = null;
		final int nf = fragments.length;
		int frank;
		for(frank = 0 ; frank != nf ; frank += 1) {
			final UmlFragment f = fragments[frank];
			if(x > f._x && y > f._y && x + w < f._x + f._w && y + h < f._y + f._h) {
				int y0 = f._y;
				final int nfc = f._compartments.length;
				int fcrank;
				for(fcrank = 0 ; fcrank != nfc ; fcrank += 1) {
					final UmlFragmentCompartment fc = f._compartments[fcrank];
					if(y > y0 && y + h < fc.b() && (r == null || fc.smaller(r))) {
						r = f._compartments[fcrank];
						break;
					}
					y0 = fc.b();
				}
			}
		}
		return r;
	}

	private String _arguments;

	private UmlFragmentCompartment[] _compartments;

	private UmlFragmentCompartment _container;

	private UmlClassInstanceReference[] _covered;

	private int _h;

	private String _name;

	private UmlDiagram _refer;

	private int _w;

	private int _x;

	private int _y;

	/**
	 * return the form corresponding to the arguments and return value of the
	 * interaction use
	 */
	public String arguments() {
		return _arguments;
	}

	/**
	 * return the compartments, at least one compartment exists
	 */
	public UmlFragmentCompartment[] compartments() {
		return _compartments;
	}

	/**
	 * return the fragment compartment containing the fragment, or 0/null
	 */
	public UmlFragmentCompartment container() {
		return _container;
	}

	/**
	 * return the the list of covered instances (life lines)
	 */
	public UmlClassInstanceReference[] covered() {
		return _covered.clone();
	}

	public int h() {
		return _h;
	}

	/**
	 * return the name
	 */
	public String name() {
		return _name;
	}

	/**
	 * internal, don't call it
	 */
	public void read_() {
		_name = UmlCom.read_string();
		_x = UmlCom.read_unsigned();
		_y = UmlCom.read_unsigned();
		_w = UmlCom.read_unsigned();
		_h = UmlCom.read_unsigned();
		final int n = UmlCom.read_unsigned();
		int rank;
		_compartments = new UmlFragmentCompartment[n];
		for(rank = 0 ; rank != n ; rank += 1) {
			final UmlFragmentCompartment fc = new UmlFragmentCompartment();
			_compartments[rank] = fc;
			fc.read_(this, rank);
		}
		_refer = (UmlDiagram)UmlItem.read_();
		_arguments = UmlCom.read_string();
	}

	/**
	 * internal, do NOT use it
	 */
	public void read_covered_(java.util.Hashtable<Integer, UmlClassInstanceReference> instances) {
		final int n = UmlCom.read_unsigned();
		int rank;
		_covered = new UmlClassInstanceReference[n];
		for(rank = 0 ; rank != n ; rank += 1) {
			_covered[rank] = instances.get(new Integer(UmlCom.read_unsigned()));
		}
	}

	/**
	 * return the diagram optionally referenced by the fragment, generally
	 * associated to an interaction use
	 */
	public UmlDiagram refer() {
		return _refer;
	}

	/**
	 * internal, do NOT use it
	 */
	public int vcenter_(int rank) {
		final int t = rank == 0 ? _y : _compartments[rank - 1].b();
		return (t + _compartments[rank].b()) / 2;
	}

	public int w() {
		return _w;
	}
}
