package org.bouml.plugout.apibase.setting;

import org.bouml.plugout.apibase.PlugoutException;
import org.bouml.plugout.apibase.UmlStereotype;
import org.bouml.plugout.apibase.com.CmdFamily;
import org.bouml.plugout.apibase.com.PhpSettingsCmd;
import org.bouml.plugout.apibase.com.UmlCom;

/**
 * This class manages settings concerning PHP, configured through the
 * 'Generation settings' dialog.
 * This class may be defined as a 'singleton', but I prefer to use static
 * members allowing to just write 'PhpSettings::member' rather than
 * 'PhpSettings::instance()->member' or other long sentence like this.
 */
public class PhpSettings extends UmlSettings {
	private static String _attr_decl;

	private static String _class_decl;

	private static boolean _defined;

	private static String _enum_decl;

	private static String _enum_item_decl;

	private static String _ext;

	private static String _external_class_decl;

	private static String _get_name;

	private static String _interface_decl;

	private static boolean _is_generate_javadoc_comment;

	private static boolean _is_get_final;

	private static boolean _is_set_final;

	private static String _oper_def;

	private static String _rel_decl;

	private static String _root;

	private static String _set_name;

	private static String _src_content;

	/**
	 * returns the default definition of an attribute
	 */
	public static String attributeDecl() {
		read_if_needed_();
		return _attr_decl;
	}

	/**
	 * returns the default definition of a class
	 */
	public static String classDecl() {
		read_if_needed_();
		return _class_decl;
	}

	/**
	 * returns the PHP stereotype corresponding to the 'UML' stereotype given in
	 * argument
	 */
	public static String classStereotype(String s) {
		read_if_needed_();
		final UmlStereotype b = UmlSettings._map_class_stereotypes.get(s);
		return b != null ? b.php : s;
	}

	/**
	 * reverse of the ClassStereotype() operation, returns the 'UML' stereotype
	 * corresponding to the PHP one given in argument
	 */
	public static String classUmlStereotype(String s) {
		read_if_needed_();
		int index = _class_stereotypes.length;
		while(index-- != 0) {
			if(_class_stereotypes[index].php.equals(s)) {
				return _class_stereotypes[index].uml;
			}
		}
		return null;
	}

	/**
	 * returns the default definition of an enumeration
	 */
	public static String enumDecl() {
		read_if_needed_();
		return _enum_decl;
	}

	/**
	 * returns the default definition of an enumeration item
	 */
	public static String enumItemDecl() {
		read_if_needed_();
		return _enum_item_decl;
	}

	/**
	 * returns the default specification for an 'external' class
	 */
	public static String externalClassDecl() {
		read_if_needed_();
		return _external_class_decl;
	}

	/**
	 * returns the default name of a 'get' operation generated through the
	 * attribute and relation 'add get operation' menu
	 */
	public static String getName() {
		read_if_needed_();
		return _get_name;
	}

	/**
	 * returns the default visibility of a 'get' operation generated through the
	 * attribute and relation 'add get operation'
	 * note : visibility shared with Java
	 */
	public static org.bouml.plugout.apibase.type.aVisibility getVisibility() {
		return JavaSettings.getVisibility();
	}

	/**
	 * returns the default definition of an interface
	 */
	public static String interfaceDecl() {
		read_if_needed_();
		return _interface_decl;
	}

	/**
	 * return if generate Javadoc style comment
	 */
	public static boolean isGenerateJavadocStyleComment() {
		read_if_needed_();
		return _is_generate_javadoc_comment;
	}

	/**
	 * return if a 'get' operation generated through the attribute and relation
	 * 'add get operation' menu is final by default
	 */
	public static boolean isGetFinal() {
		read_if_needed_();
		return _is_get_final;
	}

	/**
	 * returns if a 'set' operation generated through the attribute and relation
	 * 'add set operation' menu is final by default
	 */
	public static boolean isSetFinal() {
		read_if_needed_();
		return _is_set_final;
	}

	/**
	 * returns the default definition of an operation
	 */
	public static String operationDef() {
		read_if_needed_();
		return _oper_def;
	}

	protected static void read_() {
		_root = UmlCom.read_string();
		int n;
		int index;
		n = UmlCom.read_unsigned();
		for(index = 0 ; index != n ; index += 1) {
			UmlSettings._class_stereotypes[index].php = UmlCom.read_string();
		}
		_src_content = UmlCom.read_string();
		_ext = UmlCom.read_string();
		_class_decl = UmlCom.read_string();
		_external_class_decl = UmlCom.read_string();
		_enum_decl = UmlCom.read_string();
		_interface_decl = UmlCom.read_string();
		_attr_decl = UmlCom.read_string();
		_enum_item_decl = UmlCom.read_string();
		_rel_decl = UmlCom.read_string();
		_oper_def = UmlCom.read_string();
		UmlCom.read_char(); // getter visibility
		_get_name = UmlCom.read_string();
		_is_get_final = UmlCom.read_bool();
		UmlCom.read_char(); // setter visibility
		_set_name = UmlCom.read_string();
		_is_set_final = UmlCom.read_bool();
		_is_generate_javadoc_comment = UmlCom.read_bool();
	}

	protected static void read_if_needed_() {
		UmlSettings.read_if_needed_();
		if(!_defined) {
			UmlCom.send_cmd(CmdFamily.phpSettingsCmd, PhpSettingsCmd._getPhpSettingsCmd);
			read_();
			_defined = true;
		}
	}

	/**
	 * returns the default definition of an relation
	 */
	public static String relationDecl() {
		read_if_needed_();
		return _rel_decl;
	}

	/**
	 * return the 'root' directory
	 */
	public static String rootDir() {
		read_if_needed_();
		return _root;
	}

	/**
	 * set the default definition of an attribute
	 * On error : return FALSE in C++, produce a PlugoutException in Java
	 */
	public static void set_AttributeDecl(String v) throws PlugoutException {
		UmlCom.send_cmd(CmdFamily.phpSettingsCmd, PhpSettingsCmd._setPhpAttributeDeclCmd, v);
		UmlCom.check();
		_attr_decl = v;
	}

	/**
	 * set the default definition of a class
	 * On error : return FALSE in C++, produce a PlugoutException in Java
	 */
	public static void set_ClassDecl(String v) throws PlugoutException {
		UmlCom.send_cmd(CmdFamily.phpSettingsCmd, PhpSettingsCmd._setPhpClassDeclCmd, v);
		UmlCom.check();
		_class_decl = v;
	}

	/**
	 * set the PHP stereotype corresponding to the 'UML' stereotype given in
	 * argument
	 * On error : return FALSE in C++, produce a PlugoutException in Java
	 */
	public static void set_ClassStereotype(String s, String v) throws PlugoutException {
		read_if_needed_();
		UmlCom.send_cmd(CmdFamily.phpSettingsCmd, PhpSettingsCmd._setPhpClassStereotypeCmd, s, v);
		UmlCom.check();
		UmlStereotype st = UmlSettings._map_class_stereotypes.get(s);
		if(st == null) {
			st = UmlSettings.add_class_stereotype(s);
		}
		st.php = v;
	}

	/**
	 * set the default definition of an enumeration
	 * On error : return FALSE in C++, produce a PlugoutException in Java
	 */
	public static void set_EnumDecl(String v) throws PlugoutException {
		UmlCom.send_cmd(CmdFamily.phpSettingsCmd, PhpSettingsCmd._setPhpEnumDeclCmd, v);
		UmlCom.check();
		_enum_decl = v;
	}

	/**
	 * set the default definition of an enumeration item
	 * On error : return FALSE in C++, produce a PlugoutException in Java
	 */
	public static void set_EnumItemDecl(String v) throws PlugoutException {
		UmlCom.send_cmd(CmdFamily.phpSettingsCmd, PhpSettingsCmd._setPhpEnumItemDeclCmd, v);
		UmlCom.check();
		_enum_item_decl = v;
	}

	/**
	 * set the default specification for an 'external' class
	 * On error : return FALSE in C++, produce a PlugoutException in Java
	 */
	public static void set_ExternalClassDecl(String v) throws PlugoutException {
		UmlCom.send_cmd(CmdFamily.phpSettingsCmd, PhpSettingsCmd._setPhpExternalClassDeclCmd, v);
		UmlCom.check();
		_external_class_decl = v;
	}

	/**
	 * set the default name of a 'get' operation generated through the attribute
	 * and relation 'add get operation' menu
	 * On error : return FALSE in C++, produce a PlugoutException in Java
	 */
	public static void set_GetName(String v) throws PlugoutException {
		UmlCom.send_cmd(CmdFamily.phpSettingsCmd, PhpSettingsCmd._setPhpGetNameCmd, v);
		UmlCom.check();
		_get_name = v;
	}

	/**
	 * set the default visibility of a 'get' operation generated through the
	 * attribute and relation 'add get operation'
	 * On error : return FALSE in C++, produce a PlugoutException in Java
	 * note : visibility shared with Java
	 */
	public static void set_GetVisibility(org.bouml.plugout.apibase.type.aVisibility v) throws PlugoutException {
		JavaSettings.set_GetVisibility(v);
	}

	/**
	 * set the default definition of an interface
	 * On error : return FALSE in C++, produce a PlugoutException in Java
	 */
	public static void set_InterfaceDecl(String v) throws PlugoutException {
		UmlCom.send_cmd(CmdFamily.phpSettingsCmd, PhpSettingsCmd._setPhpInterfaceDeclCmd, v);
		UmlCom.check();
		_interface_decl = v;
	}

	/**
	 * set if generate Javadoc style comment
	 * On error : return FALSE in C++, produce a PlugoutException in Java
	 */
	public static void set_IsGenerateJavadocStyleComment(boolean v) {
		UmlCom.send_cmd(CmdFamily.phpSettingsCmd, PhpSettingsCmd._setPhpJavadocStyleCmd, v ? (byte)1 : (byte)0);
		UmlCom.check();
		_is_generate_javadoc_comment = v;
	}

	/**
	 * set if a 'get' operation generated through the attribute and relation
	 * 'add get operation' menu is final by default
	 * On error : return FALSE in C++, produce a PlugoutException in Java
	 */
	public static void set_IsGetFinal(boolean v) throws PlugoutException {
		UmlCom.send_cmd(CmdFamily.phpSettingsCmd, PhpSettingsCmd._setPhpIsGetFinalCmd, v ? (byte)1 : (byte)0);
		UmlCom.check();
		_is_get_final = v;
	}

	/**
	 * set if a 'set' operation generated through the attribute and relation
	 * 'add set operation' menu is final by default
	 * On error : return FALSE in C++, produce a PlugoutException in Java
	 */
	public static void set_IsSetFinal(boolean v) throws PlugoutException {
		UmlCom.send_cmd(CmdFamily.phpSettingsCmd, PhpSettingsCmd._setPhpIsSetFinalCmd, v ? (byte)1 : (byte)0);
		UmlCom.check();
		_is_set_final = v;
	}

	/**
	 * set the default definition of an operation
	 * On error : return FALSE in C++, produce a PlugoutException in Java
	 */
	public static void set_OperationDef(String v) throws PlugoutException {
		UmlCom.send_cmd(CmdFamily.phpSettingsCmd, PhpSettingsCmd._setPhpOperationDefCmd, v);
		UmlCom.check();
		_oper_def = v;
	}

	/**
	 * set the default definition of an relation
	 * On error : return FALSE in C++, produce a PlugoutException in Java
	 */
	public static void set_RelationDecl(String v) throws PlugoutException {
		UmlCom.send_cmd(CmdFamily.phpSettingsCmd, PhpSettingsCmd._setPhpRelationDeclCmd, v);
		UmlCom.check();
		_rel_decl = v;
	}

	/**
	 * set the 'root' directory
	 * On error : return FALSE in C++, produce a PlugoutException in Java
	 */
	public static void set_RootDir(String v) throws PlugoutException {
		UmlCom.send_cmd(CmdFamily.phpSettingsCmd, PhpSettingsCmd._setPhpRootdirCmd, v);
		UmlCom.check();
		_root = v;
	}

	/**
	 * set the default name of a 'set' operation generated through the attribute
	 * and relation 'add set operation' menu
	 * On error : return FALSE in C++, produce a PlugoutException in Java
	 */
	public static void set_SetName(String v) throws PlugoutException {
		UmlCom.send_cmd(CmdFamily.phpSettingsCmd, PhpSettingsCmd._setPhpSetNameCmd, v);
		UmlCom.check();
		_set_name = v;
	}

	/**
	 * set the default visibility of a 'set' operation generated through the
	 * attribute and relation 'add set operation'
	 * On error : return FALSE in C++, produce a PlugoutException in Java
	 * note : visibility shared with Java
	 */
	public static void set_SetVisibility(org.bouml.plugout.apibase.type.aVisibility v) throws PlugoutException {
		JavaSettings.set_SetVisibility(v);
	}

	/**
	 * set the default source file content
	 * On error : return FALSE in C++, produce a PlugoutException in Java
	 */
	public static void set_SourceContent(String v) throws PlugoutException {
		UmlCom.send_cmd(CmdFamily.phpSettingsCmd, PhpSettingsCmd._setPhpSourceContentCmd, v);
		UmlCom.check();
		_src_content = v;
	}

	/**
	 * set the extension of the file produced by the PHP code generator
	 * On error : return FALSE in C++, produce a PlugoutException in Java
	 */
	public static void set_SourceExtension(String v) throws PlugoutException {
		UmlCom.send_cmd(CmdFamily.phpSettingsCmd, PhpSettingsCmd._setPhpSourceExtensionCmd, v);
		UmlCom.check();
		_ext = v;
	}

	/**
	 * if y is TRUE the future created Php objects will be initialized with the
	 * default declaration/definition
	 * On error : return FALSE in C++, produce a PlugoutException in Java
	 */
	public static void set_UseDefaults(boolean y) throws PlugoutException {
		UmlCom.send_cmd(CmdFamily.phpSettingsCmd, PhpSettingsCmd._setPhpUseDefaultsCmd, y ? (byte)1 : (byte)0);
		UmlCom.check();
	}

	/**
	 * returns the default name of a 'set' operation generated through the
	 * attribute and relation 'add set operation' menu
	 */
	public static String setName() {
		read_if_needed_();
		return _set_name;
	}

	/**
	 * returns the default visibility of a 'set' operation generated through the
	 * attribute and relation 'add set operation'
	 * note : visibility shared with Java
	 */
	public static org.bouml.plugout.apibase.type.aVisibility setVisibility() {
		return JavaSettings.setVisibility();
	}

	/**
	 * returns the default source file content
	 */
	public static String sourceContent() {
		read_if_needed_();
		return _src_content;
	}

	/**
	 * returns the extension of the file produced by the PHP code generator
	 */
	public static String sourceExtension() {
		read_if_needed_();
		return _ext;
	}

	/**
	 * returns TRUE when the created Php objects are initialized with the
	 * default declaration/definition
	 */
	public static boolean useDefaults() {
		UmlCom.send_cmd(CmdFamily.phpSettingsCmd, PhpSettingsCmd._getPhpUseDefaultsCmd);
		return UmlCom.read_bool();
	}

	/**
	 * never called !
	 */
	private PhpSettings() {
	}
}
