package org.bouml.plugout.apibase.umlitem;

import org.bouml.plugout.apibase.PlugoutException;
import org.bouml.plugout.apibase.com.OnInstanceCmd;
import org.bouml.plugout.apibase.com.UmlCom;
import org.bouml.plugout.apibase.type.anItemKind;

public class UmlAcceptCallAction extends UmlActivityAction {
	/**
	 * returns a new accept call action named 's' created under 'parent'
	 * In case it cannot be created (the name is already used or invalid,
	 * 'parent' cannot contain it etc ...) return 0 in C++ and produce a
	 * PlugoutException in Java
	 */
	public static UmlAcceptCallAction create(UmlItem parent, String s) throws PlugoutException {
		return (UmlAcceptCallAction)parent.create_(anItemKind.anAcceptCallAction, s);
	}

	private String _cpp_trigger;

	private String _java_trigger;

	private String _trigger;

	/**
	 * the constructor, do not call it yourself !!!!!!!!!!
	 */
	protected UmlAcceptCallAction(long id, String s) {
		super(id, s);
	}

	/**
	 * return the trigger in C++
	 */
	public String cppTrigger() {
		read_if_needed_();
		return _cpp_trigger;
	}

	/**
	 * return the trigger in Java
	 */
	public String javaTrigger() {
		read_if_needed_();
		return _java_trigger;
	}

	/**
	 * returns the kind of the item
	 */
	@Override
	public anItemKind kind() {
		return anItemKind.anAcceptCallAction;
	}

	@Override
	protected void read_cpp_() {
		super.read_cpp_();
		_cpp_trigger = UmlCom.read_string();
	}

	@Override
	protected void read_java_() {
		super.read_java_();
		_java_trigger = UmlCom.read_string();
	}

	@Override
	protected void read_uml_() {
		super.read_uml_();
		_trigger = UmlCom.read_string();
	}

	/**
	 * set the trigger in C++
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_CppTrigger(String v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setCppTriggerCmd, v);
		UmlCom.check();
		_cpp_trigger = v;
	}

	/**
	 * set the trigger in Java
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_JavaTrigger(String v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setJavaTriggerCmd, v);
		UmlCom.check();
		_java_trigger = v;
	}

	/**
	 * set the trigger
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_Trigger(String v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setUmlTriggerCmd, v);
		UmlCom.check();
		_trigger = v;
	}

	/**
	 * return the trigger
	 */
	public String trigger() {
		read_if_needed_();
		return _trigger;
	}

	/**
	 * to unload the object to free memory, it will be reloaded automatically if
	 * needed. Recursively done for the sub items if 'rec' is TRUE.
	 * if 'del' is true the sub items are deleted in C++, and removed from the
	 * internal dictionnary in C++ and Java (to allow it to be garbaged), you
	 * will have to call Children() to re-access to them
	 */
	@Override
	public void unload(boolean rec, boolean del) {
		_trigger = null;
		_cpp_trigger = null;
		_java_trigger = null;
		super.unload(rec, del);
	}
}
