package org.bouml.plugout.apibase.umlitem;

import org.bouml.plugout.apibase.PlugoutException;
import org.bouml.plugout.apibase.com.OnInstanceCmd;
import org.bouml.plugout.apibase.com.UmlCom;
import org.bouml.plugout.apibase.type.anItemKind;

public class UmlAcceptEventAction extends UmlActivityAction {
	/**
	 * returns a new accept event action named 's' created under 'parent'
	 * In case it cannot be created (the name is already used or invalid,
	 * 'parent' cannot contain it etc ...) return 0 in C++ and produce a
	 * PlugoutException in Java
	 */
	public static UmlAcceptEventAction create(UmlItem parent, String s) throws PlugoutException {
		return (UmlAcceptEventAction)parent.create_(anItemKind.anAcceptEventAction, s);
	}

	private String _cpp_trigger;

	private String _java_trigger;

	private boolean _timeevent;

	private String _trigger;

	private boolean _unmarshall;

	/**
	 * the constructor, do not call it yourself !!!!!!!!!!
	 */
	protected UmlAcceptEventAction(long id, String s) {
		super(id, s);
	}

	/**
	 * return the trigger in C++
	 */
	public String cppTrigger() {
		read_if_needed_();
		return _cpp_trigger;
	}

	/**
	 * return the isTimeEvent attribute, if TRUE the event is a time event
	 */
	public boolean isTimeEvent() {
		read_if_needed_();
		return _timeevent;
	}

	/**
	 * return the isUnmarshall attribute, if TRUE there are multiple output pins
	 * for attributes or the event.
	 */
	public boolean isUnmarshall() {
		read_if_needed_();
		return _unmarshall;
	}

	/**
	 * return the trigger in Java
	 */
	public String javaTrigger() {
		read_if_needed_();
		return _java_trigger;
	}

	/**
	 * returns the kind of the item
	 */
	@Override
	public anItemKind kind() {
		return anItemKind.anAcceptEventAction;
	}

	@Override
	protected void read_cpp_() {
		super.read_cpp_();
		_cpp_trigger = UmlCom.read_string();
	}

	@Override
	protected void read_java_() {
		super.read_java_();
		_java_trigger = UmlCom.read_string();
	}

	@Override
	protected void read_uml_() {
		super.read_uml_();
		_unmarshall = UmlCom.read_bool();
		_timeevent = UmlCom.read_bool();
		_trigger = UmlCom.read_string();
	}

	/**
	 * set the trigger in C++
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_CppTrigger(String v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setCppTriggerCmd, v);
		UmlCom.check();
		_cpp_trigger = v;
	}

	/**
	 * set the isTimeEvent attribute
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_isTimeEvent(boolean v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setTimeEventCmd, v ? (byte)1 : (byte)0);
		UmlCom.check();
		_timeevent = v;
	}

	/**
	 * set the isUnmarshall attribute
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_isUnmarshall(boolean v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setUnmarshallCmd, v ? (byte)1 : (byte)0);
		UmlCom.check();
		_unmarshall = v;
	}

	/**
	 * set the trigger in Java
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_JavaTrigger(String v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setJavaTriggerCmd, v);
		UmlCom.check();
		_java_trigger = v;
	}

	/**
	 * set the trigger
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_Trigger(String v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setUmlTriggerCmd, v);
		UmlCom.check();
		_trigger = v;
	}

	/**
	 * return the trigger
	 */
	public String trigger() {
		read_if_needed_();
		return _trigger;
	}

	/**
	 * to unload the object to free memory, it will be reloaded automatically if
	 * needed. Recursively done for the sub items if 'rec' is TRUE.
	 * if 'del' is true the sub items are deleted in C++, and removed from the
	 * internal dictionnary in C++ and Java (to allow it to be garbaged), you
	 * will have to call Children() to re-access to them
	 */
	@Override
	public void unload(boolean rec, boolean del) {
		_trigger = null;
		_cpp_trigger = null;
		_java_trigger = null;
		super.unload(rec, del);
	}
}
