package org.bouml.plugout.apibase.umlitem;

import org.bouml.plugout.apibase.PlugoutException;
import org.bouml.plugout.apibase.com.OnInstanceCmd;
import org.bouml.plugout.apibase.com.UmlCom;
import org.bouml.plugout.apibase.type.anItemKind;
import org.bouml.plugout.apibase.type.anOrdering;

public class UmlActivityObject extends UmlActivityNode {
	/**
	 * returns a new activity object named 's' created under 'parent'
	 * In case it cannot be created (the name is already used or invalid,
	 * 'parent' cannot contain it etc ...) return 0 in C++ and produce a
	 * PlugoutException in Java
	 */
	public static UmlActivityObject create(UmlItem parent, String s) throws PlugoutException {
		return (UmlActivityObject)parent.create_(anItemKind.anActivityObject, s);
	}

	private UmlActivityDiagram _assoc_diagram;

	private String _cpp_selection;

	private String _in_state;

	private boolean _is_control;

	private String _java_selection;

	private String _multiplicity;

	private anOrdering _ordering;

	private String _selection;

	private UmlTypeSpec _type;

	/**
	 * the constructor, do not call it yourself !!!!!!!!!!
	 */
	protected UmlActivityObject(long id, String s) {
		super(id, s);
	}

	/**
	 * returns the optional associated diagram
	 */
	public UmlActivityDiagram associatedDiagram() {
		read_if_needed_();
		return _assoc_diagram;
	}

	/**
	 * return the selection in C++
	 */
	public String cppSelection() {
		read_if_needed_();
		return _cpp_selection;
	}

	/**
	 * return the inState attribute
	 */
	public String inState() {
		read_if_needed_();
		return _in_state;
	}

	/**
	 * return the isControlType attribute, tells whether the type of the object
	 * node is to be treated as control
	 */
	public boolean isControlType() {
		read_if_needed_();
		return _is_control;
	}

	/**
	 * return the selection in Java
	 */
	public String javaSelection() {
		read_if_needed_();
		return _java_selection;
	}

	/**
	 * returns the kind of the item
	 */
	@Override
	public anItemKind kind() {
		return anItemKind.anActivityObject;
	}

	/**
	 * return the multiplicity
	 */
	public String multiplicity() {
		read_if_needed_();
		return _multiplicity;
	}

	/**
	 * return the ordering
	 */
	public anOrdering ordering() {
		read_if_needed_();
		return _ordering;
	}

	@Override
	protected void read_cpp_() {
		_cpp_selection = UmlCom.read_string();
	}

	@Override
	protected void read_java_() {
		_java_selection = UmlCom.read_string();
	}

	@Override
	protected void read_uml_() {
		_assoc_diagram = kind() == anItemKind.anActivityObject ? (UmlActivityDiagram)UmlItem.read_() : null;
		super.read_uml_();
		_type = new UmlTypeSpec();
		_type.type = (UmlClass)UmlItem.read_();
		if(_type.type == null) {
			_type.explicit_type = UmlCom.read_string();
		}
		_multiplicity = UmlCom.read_string();
		_in_state = UmlCom.read_string();
		_selection = UmlCom.read_string();
		_ordering = anOrdering.fromInt(UmlCom.read_char());
		_is_control = UmlCom.read_bool();
	}

	/**
	 * return the selection in OCL
	 */
	public String selection() {
		read_if_needed_();
		return _selection;
	}

	/**
	 * sets the associated diagram, arg may be null to unset it
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_AssociatedDiagram(UmlActivityDiagram d) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setAssocDiagramCmd, d == null ? (long)0 : d.identifier_());
		UmlCom.check();
		_assoc_diagram = d;
	}

	/**
	 * set the selection in C++
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_CppSelection(String v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setCppActivityCmd, v);
		UmlCom.check();
		_cpp_selection = v;
	}

	/**
	 * set the inState attribute
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_InState(String v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setInStateCmd, v);
		UmlCom.check();
		_in_state = v;
	}

	/**
	 * set the isControlType attribute
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_IsControlType(boolean v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setFlagCmd, v ? 1 : 0);
		UmlCom.check();
		_is_control = v;
	}

	/**
	 * set the selection in Java
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_JavaSelection(String v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setJavaActivityCmd, v);
		UmlCom.check();
		_java_selection = v;
	}

	/**
	 * set the multiplicity
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_Multiplicity(String v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setMultiplicityCmd, v);
		UmlCom.check();
		_multiplicity = v;
	}

	/**
	 * set the ordering
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_Ordering(anOrdering v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setOrderingCmd, (byte)v.value());
		UmlCom.check();
		_ordering = v;
	}

	/**
	 * set the selection in OCL
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_Selection(String v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setUmlActivityCmd, v);
		UmlCom.check();
		_selection = v;
	}

	/**
	 * set the type
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_Type(UmlTypeSpec v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setTypeCmd, v);
		UmlCom.check();
		_type = v;
	}

	/**
	 * return the type
	 */
	public UmlTypeSpec type() {
		read_if_needed_();
		return _type;
	}

	/**
	 * to unload the object to free memory, it will be reloaded automatically if
	 * needed. Recursively done for the sub items if 'rec' is TRUE.
	 * if 'del' is true the sub items are deleted in C++, and removed from the
	 * internal dictionnary in C++ and Java (to allow it to be garbaged), you
	 * will have to call Children() to re-access to them
	 */
	@Override
	public void unload(boolean rec, boolean del) {
		_type = null;
		_multiplicity = null;
		_in_state = null;
		_selection = null;
		_cpp_selection = null;
		_java_selection = null;
		super.unload(rec, del);
	}
}
