package org.bouml.plugout.apibase.umlitem;

import org.bouml.plugout.apibase.PlugoutException;
import org.bouml.plugout.apibase.com.OnInstanceCmd;
import org.bouml.plugout.apibase.com.UmlCom;
import org.bouml.plugout.apibase.type.anItemKind;

public class UmlActivityPartition extends UmlItem {
	/**
	 * returns a new activity partition named 's' created under 'parent'
	 * In case it cannot be created (the name is already used or invalid,
	 * 'parent' cannot contain it etc ...) return 0 in C++ and produce a
	 * PlugoutException in Java
	 */
	public static UmlActivityPartition create(UmlItem parent, String s) throws PlugoutException {
		return (UmlActivityPartition)parent.create_(anItemKind.aPartition, s);
	}

	private UmlActivityDiagram _assoc_diagram;

	private boolean _dimension;

	private boolean _external;

	private UmlItem _represents;

	/**
	 * the constructor, do not call it yourself !!!!!!!!!!
	 */
	protected UmlActivityPartition(long id, String s) {
		super(id, s);
	}

	/**
	 * returns the optional associated diagram
	 */
	public UmlActivityDiagram associatedDiagram() {
		read_if_needed_();
		return _assoc_diagram;
	}

	/**
	 * return the return the isDimension attribute
	 */
	public boolean isDimension() {
		read_if_needed_();
		return _dimension;
	}

	/**
	 * return the return the isExternal attribute
	 */
	public boolean isExternal() {
		read_if_needed_();
		return _external;
	}

	@Override
	public anItemKind kind() {
		return anItemKind.aPartition;
	}

	@Override
	protected void read_uml_() {
		_assoc_diagram = (UmlActivityDiagram)UmlItem.read_();
		super.read_uml_();
		_dimension = UmlCom.read_bool();
		_external = UmlCom.read_bool();
		_represents = UmlItem.read_();
	}

	/**
	 * return the represents
	 */
	public UmlItem represents() {
		read_if_needed_();
		return _represents;
	}

	/**
	 * sets the associated diagram, arg may be null to unset it
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_AssociatedDiagram(UmlActivityDiagram d) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setAssocDiagramCmd, d == null ? (long)0 : d.identifier_());
		UmlCom.check();
		_assoc_diagram = d;
	}

	/**
	 * set the isDimension attribute
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_isDimension(boolean v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setMultiplicityCmd, v ? (byte)1 : (byte)0);
		UmlCom.check();
		_dimension = v;
	}

	/**
	 * set the isExternal attribute
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_isExternal(boolean v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setIsCppExternalCmd, v ? (byte)1 : (byte)0);
		UmlCom.check();
		_external = v;
	}

	/**
	 * set the represents
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_Represents(UmlItem v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setDerivedCmd, v == null ? (long)0 : v.identifier_());
		UmlCom.check();
		_represents = v;
	}
}
