package org.bouml.plugout.apibase.umlitem;

import org.bouml.plugout.apibase.PlugoutException;
import org.bouml.plugout.apibase.com.OnInstanceCmd;
import org.bouml.plugout.apibase.com.UmlCom;
import org.bouml.plugout.apibase.type.anItemKind;

public class UmlAddVariableValueAction extends UmlAccessVariableValueAction {
	/**
	 * returns a new add variable value action named 's' created under 'parent'
	 * In case it cannot be created (the name is already used or invalid,
	 * 'parent' cannot contain it etc ...) return 0 in C++ and produce a
	 * PlugoutException in Java
	 */
	public static UmlAddVariableValueAction create(UmlItem parent, String s) throws PlugoutException {
		return (UmlAddVariableValueAction)parent.create_(anItemKind.anAddVariableValueAction, s);
	}

	private boolean _replace_all;

	/**
	 * the constructor, do not call it yourself !!!!!!!!!!
	 */
	protected UmlAddVariableValueAction(long id, String s) {
		super(id, s);
	}

	/**
	 * return the isReplaceAll attribute, if TRUE existing values of the
	 * variable must be removed before adding the new value
	 */
	public boolean isReplaceAll() {
		read_if_needed_();
		return _replace_all;
	}

	/**
	 * returns the kind of the item
	 */
	@Override
	public anItemKind kind() {
		return anItemKind.anAddVariableValueAction;
	}

	@Override
	protected void read_uml_() {
		super.read_uml_();
		_replace_all = UmlCom.read_bool();
	}

	/**
	 * set the isReplaceAll attribute
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_isReplaceAll(boolean v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setFlagCmd, v ? (byte)1 : (byte)0);
		UmlCom.check();
		_replace_all = v;
	}
}
