package org.bouml.plugout.apibase.umlitem;

import org.bouml.plugout.apibase.PlugoutException;
import org.bouml.plugout.apibase.com.OnInstanceCmd;
import org.bouml.plugout.apibase.com.UmlCom;
import org.bouml.plugout.apibase.type.anItemKind;

/**
 * Manage the class's attributs
 */
public class UmlAttribute extends UmlClassMember {
	/**
	 * returns a new attribute named 's' created under 'parent'
	 * In case it cannot be created (the name is already used or invalid,
	 * 'parent' cannot contain it etc ...) return 0 in C++ and produce a
	 * PlugoutException in Java
	 */
	public static UmlAttribute create(UmlClass parent, String s) throws PlugoutException {
		return (UmlAttribute)parent.create_(anItemKind.anAttribute, s);
	}

	private boolean _cpp_mutable;

	private String _default_value;

	private boolean _derived;

	private boolean _derived_union;

	private UmlOperation _get_oper;

	/**
	 * exclusive with idl_explicit_case
	 */
	private UmlAttribute _idl_case;

	private String _idl_explicit_case;

	private boolean _java_transient;

	private String _multiplicity;

	private boolean _ordered;

	private boolean _read_only;

	private UmlOperation _set_oper;

	private UmlTypeSpec _type;

	private boolean _unique;

	/**
	 * the constructor, do not call it yourself !!!!!!!!!!
	 */
	protected UmlAttribute(long id, String n) {
		super(id, n);
	}

	/**
	 * to generate an associated 'get' operation
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void addGetOperation() throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.addGetOperCmd);
		UmlCom.check();
		reread_children_if_needed_();
	}

	/**
	 * to generate an associated 'set' operation
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void addSetOperation() throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.addSetOperCmd);
		UmlCom.check();
		reread_children_if_needed_();
	}

	/**
	 * returns the default attribute value, may be an empty string
	 */
	public String defaultValue() {
		read_if_needed_();
		return _default_value;
	}

	/**
	 * returns the 'get' operation of the attribute, or 0 if it does not exist
	 */
	public UmlOperation getOperation() {
		read_if_needed_();
		return _get_oper;
	}

	/**
	 * in case the attribute is an IDL union's member returns the corresponding
	 * 'case', an empty string in case it is not specified
	 */
	public String idlCase() {
		read_if_needed_();
		return _idl_case != null ? _idl_case.name() : _idl_explicit_case;
	}

	/**
	 * Indicate if the attribute is 'mutable'
	 */
	public boolean isCppMutable() {
		read_if_needed_();
		return _cpp_mutable;
	}

	/**
	 * return the property 'derived'
	 */
	public boolean isDerived() {
		read_if_needed_();
		return _derived;
	}

	/**
	 * return the property 'derived union'
	 */
	public boolean isDerivedUnion() {
		read_if_needed_();
		return _derived_union;
	}

	/**
	 * indicates if the attribute is 'transient', returns TRUE if yes
	 */
	public boolean isJavaTransient() {
		read_if_needed_();
		return _java_transient;
	}

	/**
	 * return the property 'ordered'
	 */
	public boolean isOrdered() {
		read_if_needed_();
		return _ordered;
	}

	/**
	 * indicates if the attribute is read only, returns TRUE if yes
	 */
	public boolean isReadOnly() {
		read_if_needed_();
		return _read_only;
	}

	/**
	 * return the property 'unique'
	 */
	public boolean isUnique() {
		read_if_needed_();
		return _unique;
	}

	@Override
	public anItemKind kind() {
		return anItemKind.anAttribute;
	}

	/**
	 * returns the multiplicity (may be an empty string)
	 */
	public String multiplicity() {
		read_if_needed_();
		return _multiplicity;
	}

	@Override
	protected void read_cpp_() {
		super.read_cpp_();
		_cpp_mutable = UmlCom.read_bool();
	}

	@Override
	protected void read_idl_() {
		super.read_idl_();
		_idl_case = (UmlAttribute)UmlItem.read_();
		if(_idl_case == null) {
			_idl_explicit_case = UmlCom.read_string();
		}
	}

	@Override
	protected void read_java_() {
		super.read_java_();
		_java_transient = UmlCom.read_bool();
	}

	@Override
	protected void read_php_() {
		super.read_php_();
	}

	@Override
	protected void read_python_() {
		super.read_python_();
	}

	@Override
	protected void read_uml_() {
		super.read_uml_();
		_type = new UmlTypeSpec();
		_type.type = (UmlClass)UmlItem.read_();
		if(_type.type == null) {
			_type.explicit_type = UmlCom.read_string();
		}
		_multiplicity = UmlCom.read_string();
		_default_value = UmlCom.read_string();
		_read_only = UmlCom.read_bool();
		_derived = UmlCom.read_bool();
		_derived_union = UmlCom.read_bool();
		_ordered = UmlCom.read_bool();
		_unique = UmlCom.read_bool();
		_get_oper = (UmlOperation)UmlItem.read_();
		_set_oper = (UmlOperation)UmlItem.read_();
	}

	/**
	 * to set the default attribute value ("" allowed)
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_DefaultValue(String s) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setDefaultValueCmd, s);
		UmlCom.check();
		_default_value = s;
	}

	/**
	 * to set the 'case' even the attribute is not (already) known as an IDL
	 * union's member
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_IdlCase(String s) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setIdlCaseCmd, 0, s);
		UmlCom.check();
		_idl_case = null;
		_idl_explicit_case = s;
	}

	/**
	 * to set the 'case' even the attribute is not (already) known as an IDL
	 * union's member
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_IdlCase(UmlAttribute a) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setIdlCaseCmd, a.identifier_(), "");
		UmlCom.check();
		_idl_case = a;
		_idl_explicit_case = null;
	}

	/**
	 * Set if the attribute is 'mutable'
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_isCppMutable(boolean y) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setIsCppMutableCmd, y ? (byte)1 : (byte)0);
		UmlCom.check();
		_cpp_mutable = y;
	}

	/**
	 * Set the properties 'derived' and 'union'
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_isDerived(boolean is_derived, boolean is_union) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setDerivedCmd, (byte)((is_derived ? 1 : 0) + (is_union ? 2 : 0)));
		UmlCom.check();
		_derived = is_derived;
		_derived_union = is_union;
	}

	/**
	 * to set the 'transient' state of the attribute
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_isJavaTransient(boolean y) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setIsJavaTransientCmd, y ? (byte)1 : (byte)0);
		UmlCom.check();
		_java_transient = y;
	}

	/**
	 * set the property 'ordered'
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_isOrdered(boolean v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setOrderingCmd, v ? 1 : 0);
		UmlCom.check();
		_ordered = v;
	}

	/**
	 * to set the 'read only' state of the attribute
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_isReadOnly(boolean y) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setIsReadOnlyCmd, y ? (byte)1 : (byte)0);
		UmlCom.check();
		_read_only = y;
	}

	/**
	 * set the property 'unique'
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_isUnique(boolean v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setUniqueCmd, v ? 1 : 0);
		UmlCom.check();
		_unique = v;
	}

	/**
	 * to set the multiplicity
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_Multiplicity(String s) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setMultiplicityCmd, s);
		UmlCom.check();
		_multiplicity = s;
	}

	/**
	 * to set the attribute UML type
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_Type(UmlTypeSpec t) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setTypeCmd, t);
		UmlCom.check();
		_type = t;
	}

	/**
	 * returns the 'set' operation of the attribute, or 0 if it does not exist
	 */
	public UmlOperation setOperation() {
		read_if_needed_();
		return _set_oper;
	}

	/**
	 * returns the attribute UML type
	 */
	public UmlTypeSpec type() {
		read_if_needed_();
		return _type;
	}

	@Override
	public void unload(boolean rec, boolean del) {
		_type = null;
		_default_value = null;
		_idl_explicit_case = null;
		super.unload(rec, del);
		_multiplicity = null;
	}
}
