package org.bouml.plugout.apibase.umlitem;

import org.bouml.plugout.apibase.PlugoutException;
import org.bouml.plugout.apibase.com.OnInstanceCmd;
import org.bouml.plugout.apibase.com.UmlCom;
import org.bouml.plugout.apibase.type.anItemKind;

public class UmlCallBehaviorAction extends UmlActivityAction {
	/**
	 * returns a new call behavior action named 's' created under 'parent'
	 * In case it cannot be created (the name is already used or invalid,
	 * 'parent' cannot contain it etc ...) return 0 in C++ and produce a
	 * PlugoutException in Java
	 */
	public static UmlCallBehaviorAction create(UmlItem parent, String s) throws PlugoutException {
		return (UmlCallBehaviorAction)parent.create_(anItemKind.aCallBehaviorAction, s);
	}

	private UmlItem _behavior;

	private boolean _synchronous;

	/**
	 * the constructor, do not call it yourself !!!!!!!!!!
	 */
	protected UmlCallBehaviorAction(long id, String s) {
		super(id, s);
	}

	/**
	 * return the behavior, may be an activity or a state machine
	 */
	public UmlItem behavior() {
		read_if_needed_();
		return _behavior;
	}

	/**
	 * return the isSynchronous attribute, if TRUE the caller waits for the
	 * completion of the invoked behavior
	 */
	public boolean isSynchronous() {
		read_if_needed_();
		return _synchronous;
	}

	/**
	 * returns the kind of the item
	 */
	@Override
	public anItemKind kind() {
		return anItemKind.aCallBehaviorAction;
	}

	@Override
	protected void read_uml_() {
		super.read_uml_();
		_synchronous = UmlCom.read_bool();
		_behavior = UmlItem.read_();
	}

	/**
	 * set the behavior
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_Behavior(UmlItem v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setDefCmd, v.identifier_());
		UmlCom.check();
		_behavior = v;
	}

	/**
	 * set the isSynchronous attribute
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_isSynchronous(boolean v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setFlagCmd, v ? (byte)1 : (byte)0);
		UmlCom.check();
		_synchronous = v;
	}
}
