package org.bouml.plugout.apibase.umlitem;

import org.bouml.plugout.apibase.PlugoutException;
import org.bouml.plugout.apibase.com.OnInstanceCmd;
import org.bouml.plugout.apibase.com.UmlCom;
import org.bouml.plugout.apibase.type.anItemKind;

public class UmlCallOperationAction extends UmlActivityAction {
	/**
	 * returns a new call operation action named 's' created under 'parent'
	 * In case it cannot be created (the name is already used or invalid,
	 * 'parent' cannot contain it etc ...) return 0 in C++ and produce a
	 * PlugoutException in Java
	 */
	public static UmlCallOperationAction create(UmlItem parent, String s) throws PlugoutException {
		return (UmlCallOperationAction)parent.create_(anItemKind.aCallOperationAction, s);
	}

	private UmlOperation _operation;

	private boolean _synchronous;

	/**
	 * the constructor, do not call it yourself !!!!!!!!!!
	 */
	protected UmlCallOperationAction(long id, String s) {
		super(id, s);
	}

	/**
	 * return the isSynchronous attribute, if TRUE the caller waits for the
	 * completion of the invoked behavior
	 */
	public boolean isSynchronous() {
		read_if_needed_();
		return _synchronous;
	}

	/**
	 * returns the kind of the item
	 */
	@Override
	public anItemKind kind() {
		return anItemKind.aCallOperationAction;
	}

	/**
	 * return the operation
	 */
	public UmlOperation operation() {
		read_if_needed_();
		return _operation;
	}

	@Override
	protected void read_uml_() {
		super.read_uml_();
		_synchronous = UmlCom.read_bool();
		_operation = (UmlOperation)UmlItem.read_();
	}

	/**
	 * set the isSynchronous attribute
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_isSynchronous(boolean v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setFlagCmd, v ? (byte)1 : (byte)0);
		UmlCom.check();
		_synchronous = v;
	}

	/**
	 * set the operation
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_Operation(UmlOperation v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setDefCmd, v.identifier_());
		UmlCom.check();
		_operation = v;
	}
}
