package org.bouml.plugout.apibase.umlitem;

import java.util.Hashtable;
import org.bouml.plugout.apibase.PlugoutException;
import org.bouml.plugout.apibase.com.ClassGlobalCmd;
import org.bouml.plugout.apibase.com.CmdFamily;
import org.bouml.plugout.apibase.com.OnInstanceCmd;
import org.bouml.plugout.apibase.com.PackageGlobalCmd;
import org.bouml.plugout.apibase.com.UmlCom;
import org.bouml.plugout.apibase.type.aVisibility;
import org.bouml.plugout.apibase.type.anItemKind;

/**
 * Manage the classes
 */
public class UmlClass extends UmlClassMember {
	private static Hashtable<String, UmlClass> _classes = new Hashtable<String, UmlClass>(1001);

	/**
	 * returns a new class named 's' created under 'parent'
	 * In case it cannot be created (the name is already used or invalid,
	 * 'parent' cannot contain it etc ...) return 0 in C++ and produce a
	 * PlugoutException in Java
	 */
	public static UmlClass create(UmlItem parent, String s) throws PlugoutException {
		return (UmlClass)parent.create_(anItemKind.aClass, s);
	}

	/**
	 * Return the class supporting the stereotype corresponding to the first
	 * parameter being 'profile_name:stereotype_name', or 0/null
	 */
	public static UmlClass findStereotype(String s, boolean caseSensitive) {
		UmlCom.send_cmd(CmdFamily.packageGlobalCmd, PackageGlobalCmd._findStereotypeCmd, caseSensitive ? "y" : "n", s);
		return (UmlClass)UmlItem.read_();
	}

	/**
	 * returns the class having the name given in argument in case it exist,
	 * else 0/null. In case the package is specified (not 0/null), the returned
	 * class will be defined in a sub-level of the package.
	 */
	public static UmlClass get(String n, UmlPackage p) {
		if(p == null) {
			final UmlClass x = _classes.get(n);
			if(x != null) {
				return x;
			}
		}
		UmlCom.send_cmd(CmdFamily.classGlobalCmd, ClassGlobalCmd._findClassCmd, p != null ? ((UmlItem)p).identifier_() : 0, n);
		return (UmlClass)UmlItem.read_();
	}

	private boolean _abstract;

	private boolean _active;

	private UmlClassDiagram _assoc_diagram;

	private UmlTypeSpec _base_type;

	private boolean _cpp_external;

	private boolean _idl_custom;

	private boolean _idl_external;

	private boolean _idl_local;

	private boolean _java_external;

	private boolean _java_final;

	private boolean _php_external;

	private boolean _php_final;

	private boolean _python_2_2;

	private boolean _python_external;

	/**
	 * union
	 */
	private UmlTypeSpec _switch_type;

	/**
	 * the constructor, do not call it yourself !!!!!!!!!!
	 */
	protected UmlClass(long id, String n) {
		super(id, n);
	}

	/**
	 * returns (a copy of) the actuals list
	 */
	public UmlActualParameter[] actuals() {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.actualsCmd);
		final int n = UmlCom.read_unsigned();
		final UmlActualParameter[] actuals = new UmlActualParameter[n];
		for(int i = 0 ; i != n ; i += 1) {
			final UmlActualParameter a = new UmlActualParameter();
			a.read_();
			actuals[i] = a;
		}
		return actuals;
	}

	/**
	 * adds a formal at the given rank (0...), returns 0 on error
	 * On error return FALSE in C++, produce a PlugoutException in Java, does
	 * not check that the class is (already) a typedef
	 */
	public void addFormal(int rank, UmlFormalParameter formal) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.addFormalCmd, rank, formal._name, formal._type, formal._default_value, formal._extends);
		UmlCom.check();
	}

	/**
	 * returns the optional associated artifact, to set it refer to the
	 * UmlBaseArtifact's operations addAssociatedClass(),
	 * removeAssociatedClass() and setAssociatedClasses()
	 */
	public UmlArtifact associatedArtifact() {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.assocArtifactCmd);
		return (UmlArtifact)UmlItem.read_();
	}

	/**
	 * returns the components realizing or providing the class. To set them
	 * refer to the UmlBaseComponent's operation setAssociatedClasses()
	 */
	public UmlComponent[] associatedComponents() {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.assocComponentCmd);
		final int n = UmlCom.read_unsigned();
		final UmlComponent[] result = new UmlComponent[n];
		for(int index = 0 ; index != n ; index += 1) {
			result[index] = (UmlComponent)UmlItem.read_();
		}
		return result;
	}

	/**
	 * returns the optional associated diagram
	 */
	public UmlClassDiagram associatedDiagram() {
		read_if_needed_();
		return _assoc_diagram;
	}

	/**
	 * returns the type on which the class (in fact a typedef) is based
	 */
	public UmlTypeSpec baseType() {
		read_if_needed_();
		return _base_type;
	}

	/**
	 * returns (a copy of) the formals list
	 */
	public UmlFormalParameter[] formals() {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.formalsCmd);
		final int n = UmlCom.read_unsigned();
		final UmlFormalParameter[] formals = new UmlFormalParameter[n];
		for(int i = 0 ; i != n ; i += 1) {
			final UmlFormalParameter f = new UmlFormalParameter();
			f.read_();
			formals[i] = f;
		}
		return formals;
	}

	/**
	 * returns TRUE in case the class is abstract
	 */
	public boolean isAbstract() {
		read_if_needed_();
		return _abstract;
	}

	/**
	 * return the indicate if the class is active
	 */
	public boolean isActive() {
		read_if_needed_();
		return _active;
	}

	/**
	 * returns TRUE if the class is external, its definition must contains how
	 * the name is made on the first line (isCppExternal by default), the other
	 * lines if they exist must contains the #include and may be using forms
	 */
	public boolean isCppExternal() {
		read_if_needed_();
		return _cpp_external;
	}

	/**
	 * returns TRUE is the class is custom, have sense for a valuetype
	 */
	public boolean isIdlCustom() {
		read_if_needed_();
		return _idl_custom;
	}

	/**
	 * returns TRUE if the class is external, its definition must contain how
	 * the name is made on the first line (isIdlExternal by default), the other
	 * lines are ignored
	 */
	public boolean isIdlExternal() {
		read_if_needed_();
		return _idl_external;
	}

	/**
	 * returns TRUE if the class is local, have sense for an interface
	 */
	public boolean isIdlLocal() {
		read_if_needed_();
		return _idl_local;
	}

	/**
	 * returns TRUE if the class is external, its definition must contain how
	 * the name is made on the first line (isJavaExternal by default), the other
	 * lines are ignored
	 */
	public boolean isJavaExternal() {
		read_if_needed_();
		return _java_external;
	}

	/**
	 * returns TRUE is the class is final
	 */
	public boolean isJavaFinal() {
		read_if_needed_();
		return _java_final;
	}

	/**
	 * Returns TRUE is the class is package.
	 */
	public boolean isJavaPackage() {
		return visibility() == aVisibility.PackageVisibility;
	}

	/**
	 * Returns TRUE is the class is private.
	 */
	public boolean isJavaPrivate() {
		return visibility() == aVisibility.PrivateVisibility;
	}

	/**
	 * Returns TRUE is the class is protected.
	 */
	public boolean isJavaProtected() {
		return visibility() == aVisibility.ProtectedVisibility;
	}

	/**
	 * Returns TRUE is the class is public.
	 */
	public boolean isJavaPublic() {
		return visibility() == aVisibility.PublicVisibility;
	}

	/**
	 * returns TRUE if the class is external, its definition must contain how
	 * the name is made on the first line (isPhpExternal by default), the other
	 * lines are ignored
	 */
	public boolean isPhpExternal() {
		read_if_needed_();
		return _php_external;
	}

	/**
	 * returns TRUE is the class is final
	 */
	public boolean isPhpFinal() {
		read_if_needed_();
		return _php_final;
	}

	/**
	 * returns TRUE is the class is a Python 2.2 class
	 */
	public boolean isPython_2_2() {
		read_if_needed_();
		return _python_2_2;
	}

	/**
	 * returns TRUE if the class is external, its definition must contain how
	 * the name is made on the first line (isPythonExternal by default), the
	 * other lines are ignored
	 */
	public boolean isPythonExternal() {
		read_if_needed_();
		return _python_external;
	}

	@Override
	public anItemKind kind() {
		return anItemKind.aClass;
	}

	@Override
	protected void read_cpp_() {
		super.read_cpp_();
		_cpp_external = UmlCom.read_bool();
	}

	@Override
	protected void read_idl_() {
		super.read_idl_();
		_switch_type = new UmlTypeSpec();
		_switch_type.type = (UmlClass)UmlItem.read_();
		if(_switch_type.type == null) {
			_switch_type.explicit_type = UmlCom.read_string();
		}
		_idl_local = UmlCom.read_bool();
		_idl_custom = UmlCom.read_bool();
		_idl_external = UmlCom.read_bool();
	}

	@Override
	protected void read_java_() {
		super.read_java_();
		_java_final = UmlCom.read_bool();
		_java_external = UmlCom.read_bool();
	}

	@Override
	protected void read_php_() {
		super.read_php_();
		_php_final = UmlCom.read_bool();
		_php_external = UmlCom.read_bool();
	}

	@Override
	protected void read_python_() {
		super.read_python_();
		_python_2_2 = UmlCom.read_bool();
		_python_external = UmlCom.read_bool();
	}

	@Override
	protected void read_uml_() {
		_assoc_diagram = (UmlClassDiagram)UmlItem.read_();
		super.read_uml_();
		_abstract = UmlCom.read_bool();
		if(_stereotype.equals("typedef")) {
			_base_type = new UmlTypeSpec();
			_base_type.type = (UmlClass)UmlItem.read_();
			if(_base_type.type == null) {
				_base_type.explicit_type = UmlCom.read_string();
			}
		}
		_active = UmlCom.read_bool();
	}

	/**
	 * remove the formal of the given rank (0...), returns 0 on error
	 * On error return FALSE in C++, produce a PlugoutException in Java, does
	 * not check that the class is (already) a typedef
	 */
	public void removeFormal(int rank) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.removeFormalCmd, rank);
		UmlCom.check();
	}

	/**
	 * replace the actual value at the given rank (0...)
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void replaceActual(int rank, UmlTypeSpec type) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.replaceActualCmd, rank, type);
		UmlCom.check();
	}

	/**
	 * replace the formal at the given rank (0...)
	 * On error return FALSE in C++, produce a PlugoutException in Java, does
	 * not check that the class is (already) a typedef
	 */
	public void replaceFormal(int rank, UmlFormalParameter formal) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.replaceFormalCmd, rank, formal._name, formal._type, formal._default_value, formal._extends);
		UmlCom.check();
	}

	/**
	 * internal, do NOT use it
	 */
	protected void reread_if_needed_() {
		if(defined_()) {
			UmlCom.send_cmd(identifier_(), OnInstanceCmd.getUmlDefCmd);
			read_uml_();
		}
	}

	/**
	 * sets the associated diagram, arg may be null to unset it
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_AssociatedDiagram(UmlClassDiagram d) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setAssocDiagramCmd, d == null ? (long)0 : d.identifier_());
		UmlCom.check();
		_assoc_diagram = d;
	}

	/**
	 * to set the type on which the class (in fact a typedef) is based
	 * On error return FALSE in C++, produce a PlugoutException in Java, does
	 * not check that the class is (already) a typedef
	 */
	public void set_BaseType(UmlTypeSpec t) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setBaseTypeCmd, t);
		UmlCom.check();
		_base_type = t;
	}

	/**
	 * the set if the class is abstract
	 * On error return FALSE in C++, produce a PlugoutException in Java, for
	 * instance you cannot set it to FALSE in case the class have abstract
	 * operations
	 */
	public void set_isAbstract(boolean y) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setIsAbstractCmd, y ? (byte)1 : (byte)0);
		UmlCom.check();
		_abstract = y;
	}

	/**
	 * set if the class is active
	 * On error produce a RuntimeException in Java
	 */
	public void set_isActive(boolean v) throws RuntimeException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setActiveCmd, v ? 1 : 0);
		UmlCom.check();
		_active = v;
	}

	/**
	 * set if the class is external
	 * On error produce a PlugoutException in Java
	 */
	public void set_isCppExternal(boolean y) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setIsCppExternalCmd, y ? (byte)1 : (byte)0);
		UmlCom.check();
		_cpp_external = y;
	}

	/**
	 * set if the class is custom have sense for aa valuetype
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_isIdlCustom(boolean y) {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setIsIdlCustomCmd, y ? (byte)1 : (byte)0);
		UmlCom.check();
		_idl_custom = y;
	}

	/**
	 * set if the class is external
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_isIdlExternal(boolean y) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setIsIdlExternalCmd, y ? (byte)1 : (byte)0);
		UmlCom.check();
		_idl_external = y;
	}

	/**
	 * set if the class is local, have sense for an interface
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_isIdlLocal(boolean y) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setIsIdlLocalCmd, y ? (byte)1 : (byte)0);
		UmlCom.check();
		_idl_local = y;
	}

	/**
	 * set if the class is external
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_isJavaExternal(boolean y) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setIsJavaExternalCmd, y ? (byte)1 : (byte)0);
		UmlCom.check();
		_java_external = y;
	}

	/**
	 * set if the class is final
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_isJavaFinal(boolean y) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setIsJavaFinalCmd, y ? (byte)1 : (byte)0);
		UmlCom.check();
		_java_final = y;
	}

	/**
	 * Set if the class is public.
	 */
	public void set_isJavaPublic(boolean y) throws PlugoutException {
		set_Visibility(y ? aVisibility.PublicVisibility : aVisibility.PackageVisibility);
	}

	/**
	 * set if the class is external
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_isPhpExternal(boolean y) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setIsPhpExternalCmd, y ? (byte)1 : (byte)0);
		UmlCom.check();
		_php_external = y;
	}

	/**
	 * set if the class is final
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_isPhpFinal(boolean y) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setIsPhpFinalCmd, y ? (byte)1 : (byte)0);
		UmlCom.check();
		_php_final = y;
	}

	/**
	 * set if the class is a Python 2.2 class
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_isPython_2_2(boolean v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setIsPython2_2Cmd, v ? 1 : 0);
		UmlCom.check();
		_python_2_2 = v;
	}

	/**
	 * set if the class is external
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_isPythonExternal(boolean y) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setIsPythonExternalCmd, y ? (byte)1 : (byte)0);
		UmlCom.check();
		_python_external = y;
	}

	/**
	 * to set the name
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	@Override
	public void set_Name(String s) throws PlugoutException {
		super.set_Name(s);
		final UmlItem[] ch = children();
		final String destr = "~" + name();
		for(int i = 0 ; i != ch.length ; i += 1) {
			if(ch[i].kind() == anItemKind.anOperation) {
				if(ch[i].name() == name()) {
					ch[i].set_Name(s);
				} else if(ch[i].name() == destr) {
					ch[i].set_Name("~" + s);
				}
			}
		}
	}

	/**
	 * to set the switch type even the class is not (already) an IDL union
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_SwitchType(UmlTypeSpec t) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setSwitchTypeCmd, t);
		UmlCom.check();
		_switch_type = t.clone_it();
	}

	/**
	 * returns the switch's type, significant in case the class is an union in
	 * IDL
	 */
	public UmlTypeSpec switchType() {
		read_if_needed_();
		return _switch_type.clone_it();
	}

	/**
	 * to unload the object to free memory, it will be reloaded automatically if
	 * needed. Recursively done for the sub items if 'rec' is TRUE.
	 * if 'del' is true the sub items are deleted in C++, and removed from the
	 * internal dictionnary in C++ and Java (to allow it to be garbaged), you
	 * will have to call Children() to re-access to them
	 */
	@Override
	public void unload(boolean rec, boolean del) {
		_base_type = null;
		_switch_type = null;
		super.unload(rec, del);
	}
}
