package org.bouml.plugout.apibase.umlitem;

import org.bouml.plugout.apibase.PlugoutException;
import org.bouml.plugout.apibase.com.OnInstanceCmd;
import org.bouml.plugout.apibase.com.UmlCom;

/**
 * Mother class of the all the class's items including the class themself
 */
public abstract class UmlClassItem extends UmlItem {
	private String _cpp_decl;

	private String _idl_decl;

	private String _java_decl;

	private String _php_decl;

	private String _python_decl;

	protected UmlClassItem(long id, String n) {
		super(id, n);
	}

	/**
	 * return the C++ declaration
	 */
	public String cppDecl() {
		read_if_needed_();
		return _cpp_decl;
	}

	/**
	 * return the IDL declaration
	 */
	public String idlDecl() {
		read_if_needed_();
		return _idl_decl;
	}

	/**
	 * return the Java defininition
	 */
	public String javaDecl() {
		read_if_needed_();
		return _java_decl;
	}

	/**
	 * return the Php defininition
	 */
	public String phpDecl() {
		read_if_needed_();
		return _php_decl;
	}

	/**
	 * return the Python defininition
	 */
	public String pythonDecl() {
		read_if_needed_();
		return _python_decl;
	}

	/**
	 * internal, do NOT use it
	 */
	@Override
	protected void read_cpp_() {
		_cpp_decl = UmlCom.read_string();
	}

	/**
	 * internal, do NOT use it
	 */
	@Override
	protected void read_idl_() {
		_idl_decl = UmlCom.read_string();
	}

	/**
	 * internal, do NOT use it
	 */
	@Override
	protected void read_java_() {
		_java_decl = UmlCom.read_string();
	}

	/**
	 * internal, do NOT use it
	 */
	@Override
	protected void read_php_() {
		_php_decl = UmlCom.read_string();
	}

	/**
	 * internal, do NOT use it
	 */
	@Override
	protected void read_python_() {
		_python_decl = UmlCom.read_string();
	}

	/**
	 * to set the C++ declaration
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_CppDecl(String s) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setCppDeclCmd, s);
		UmlCom.check();
		_cpp_decl = s;
	}

	/**
	 * set the IDL declaration
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_IdlDecl(String s) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setIdlDeclCmd, s);
		UmlCom.check();
		_idl_decl = s;
	}

	/**
	 * to set the Java definition
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_JavaDecl(String s) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setJavaDeclCmd, s);
		UmlCom.check();
		_java_decl = s;
	}

	/**
	 * to set the Php definition
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_PhpDecl(String s) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setPhpDeclCmd, s);
		UmlCom.check();
		_php_decl = s;
	}

	/**
	 * to set the Python definition
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_PythonDecl(String s) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setPythonDeclCmd, s);
		UmlCom.check();
		_python_decl = s;
	}

	@Override
	public void unload(boolean rec, boolean del) {
		_cpp_decl = null;
		_java_decl = null;
		_php_decl = null;
		_python_decl = null;
		_idl_decl = null;
		super.unload(rec, del);
	}
}
