package org.bouml.plugout.apibase.umlitem;

import org.bouml.plugout.apibase.PlugoutException;
import org.bouml.plugout.apibase.com.OnInstanceCmd;
import org.bouml.plugout.apibase.com.UmlCom;
import org.bouml.plugout.apibase.type.anItemKind;

/**
 * Manage the class views
 */
public class UmlClassView extends UmlView {
	/**
	 * returns a new class view named 's' created under 'parent'
	 * In case it cannot be created (the name is already used or invalid,
	 * 'parent' cannot contain it etc ...) return 0 in C++ and produce a
	 * PlugoutException in Java
	 */
	public static UmlClassView create(UmlPackage parent, String s) throws PlugoutException {
		return (UmlClassView)parent.create_(anItemKind.aClassView, s);
	}

	private UmlDeploymentView _assoc_view;

	/**
	 * the constructor, do not call it yourself !!!!!!!!!!
	 */
	protected UmlClassView(long id, String n) {
		super(id, n);
	}

	/**
	 * returns the optional associated deployment view
	 */
	public UmlDeploymentView associatedDeploymentView() {
		read_if_needed_();
		return _assoc_view;
	}

	@Override
	public anItemKind kind() {
		return anItemKind.aClassView;
	}

	@Override
	protected void read_uml_() {
		_assoc_view = (UmlDeploymentView)UmlItem.read_();
		super.read_uml_();
	}

	/**
	 * sets the associated deployment view, arg may be null to unset it
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_AssociatedDeploymentView(UmlDeploymentView v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setAssocViewCmd, v.identifier_());
		UmlCom.check();
		_assoc_view = v;
	}
}
