package org.bouml.plugout.apibase.umlitem;

import org.bouml.plugout.apibase.PlugoutException;
import org.bouml.plugout.apibase.com.OnInstanceCmd;
import org.bouml.plugout.apibase.com.UmlCom;
import org.bouml.plugout.apibase.type.anItemKind;

/**
 * Manage the components.
 */
public class UmlComponent extends UmlItem {
	/**
	 * returns a new component named 's' created under 'parent'
	 * In case it cannot be created (the name is already used or invalid,
	 * 'parent' cannot contain it etc ...) return 0 in C++ and produce a
	 * PlugoutException in Java
	 */
	public static UmlComponent create(UmlComponentView parent, String s) throws PlugoutException {
		return (UmlComponent)parent.create_(anItemKind.aComponent, s);
	}

	private UmlComponentDiagram _assoc_diagram;

	private UmlClass[] _provided;

	private UmlClass[] _realizing;

	private UmlClass[] _required;

	/**
	 * the constructor, do not call it yourself !!!!!!!!!!
	 */
	protected UmlComponent(long id, String n) {
		super(id, n);
	}

	/**
	 * returns the optional associated diagram
	 */
	public UmlComponentDiagram associatedDiagram() {
		read_if_needed_();
		return _assoc_diagram;
	}

	@Override
	public anItemKind kind() {
		return anItemKind.aComponent;
	}

	/**
	 * returns (in Java a copy of) the optional provided classes
	 */
	public UmlClass[] providedClasses() {
		read_if_needed_();
		return _provided.clone();
	}

	@Override
	protected void read_uml_() {
		_assoc_diagram = (UmlComponentDiagram)UmlItem.read_();
		super.read_uml_();
		int n;
		int index;
		n = UmlCom.read_unsigned();
		_realizing = new UmlClass[n];
		for(index = 0 ; index != n ; index += 1) {
			_realizing[index] = (UmlClass)UmlItem.read_();
		}
		n = UmlCom.read_unsigned();
		_provided = new UmlClass[n];
		for(index = 0 ; index != n ; index += 1) {
			_provided[index] = (UmlClass)UmlItem.read_();
		}
		n = UmlCom.read_unsigned();
		_required = new UmlClass[n];
		for(index = 0 ; index != n ; index += 1) {
			_required[index] = (UmlClass)UmlItem.read_();
		}
	}

	/**
	 * returns (in Java a copy of) the optional realizing classes
	 */
	public UmlClass[] realizingClasses() {
		read_if_needed_();
		return _realizing.clone();
	}

	/**
	 * returns (in Java a copy of) the optional required classes
	 */
	public UmlClass[] requiredClasses() {
		read_if_needed_();
		return _required.clone();
	}

	/**
	 * set the realizing, provided and required classes lists
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_AssociatedClasses(UmlClass[] realizing, UmlClass[] provided, UmlClass[] required) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setAssocClassesCmd, realizing, provided, required);
		UmlCom.check();
		_realizing = realizing.clone();
		_provided = provided.clone();
		_required = required.clone();
	}

	/**
	 * sets the associated diagram, arg may be null to unset it
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_AssociatedDiagram(UmlComponentDiagram d) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setAssocDiagramCmd, d == null ? (long)0 : d.identifier_());
		UmlCom.check();
		_assoc_diagram = d;
	}

	/**
	 * to unload the object to free memory, it will be reloaded automatically if
	 * needed. args unused
	 */
	@Override
	public void unload(boolean rec, boolean del) {
		_realizing = null;
		_provided = null;
		_required = null;
		super.unload(rec, del);
	}
}
