package org.bouml.plugout.apibase.umlitem;

import org.bouml.plugout.apibase.PlugoutException;
import org.bouml.plugout.apibase.com.OnInstanceCmd;
import org.bouml.plugout.apibase.com.UmlCom;
import org.bouml.plugout.apibase.type.anItemKind;

public class UmlCreateObjectAction extends UmlActivityAction {
	/**
	 * returns a new create object action named 's' created under 'parent'
	 * In case it cannot be created (the name is already used or invalid,
	 * 'parent' cannot contain it etc ...) return 0 in C++ and produce a
	 * PlugoutException in Java
	 */
	public static UmlCreateObjectAction create(UmlItem parent, String s) throws PlugoutException {
		return (UmlCreateObjectAction)parent.create_(anItemKind.aCreateObjectAction, s);
	}

	private String _classifier;

	/**
	 * the constructor, do not call it yourself !!!!!!!!!!
	 */
	protected UmlCreateObjectAction(long id, String s) {
		super(id, s);
	}

	/**
	 * return the classifier
	 */
	public String classifier() {
		read_if_needed_();
		return _classifier;
	}

	@Override
	public anItemKind kind() {
		return anItemKind.aCreateObjectAction;
	}

	@Override
	protected void read_uml_() {
		super.read_uml_();
		_classifier = UmlCom.read_string();
	}

	/**
	 * set the classifier
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_Classifier(String v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setDefCmd, v);
		UmlCom.check();
		_classifier = v;
	}

	@Override
	public void unload(boolean rec, boolean del) {
		_classifier = null;
		super.unload(rec, del);
	}
}
