package org.bouml.plugout.apibase.umlitem;

import org.bouml.plugout.apibase.PlugoutException;
import org.bouml.plugout.apibase.com.OnInstanceCmd;
import org.bouml.plugout.apibase.com.UmlCom;
import org.bouml.plugout.apibase.type.anItemKind;

public class UmlDestroyObjectAction extends UmlActivityAction {
	/**
	 * returns a new destroy object action named 's' created under 'parent'
	 * In case it cannot be created (the name is already used or invalid,
	 * 'parent' cannot contain it etc ...) return 0 in C++ and produce a
	 * PlugoutException in Java
	 */
	public static UmlDestroyObjectAction create(UmlItem parent, String s) throws PlugoutException {
		return (UmlDestroyObjectAction)parent.create_(anItemKind.aDestroyObjectAction, s);
	}

	private boolean _links;

	private boolean _owned_objects;

	/**
	 * the constructor, do not call it yourself !!!!!!!!!!
	 */
	protected UmlDestroyObjectAction(long id, String s) {
		super(id, s);
	}

	/**
	 * return the return the isDestroyLinks attribute.
	 */
	public boolean isDestroyLinks() {
		read_if_needed_();
		return _links;
	}

	/**
	 * return the return the isDestroyOwnedObjects attribute
	 */
	public boolean isDestroyOwnedObjects() {
		read_if_needed_();
		return _owned_objects;
	}

	@Override
	public anItemKind kind() {
		return anItemKind.aDestroyObjectAction;
	}

	@Override
	protected void read_uml_() {
		super.read_uml_();
		_links = UmlCom.read_bool();
		_owned_objects = UmlCom.read_bool();
	}

	/**
	 * set the isDestroyLinks attribute
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_isDestroyLinks(boolean v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setTypeCmd, v ? (byte)1 : (byte)0);
		UmlCom.check();
		_links = v;
	}

	/**
	 * set the isDestroyOwnedObjects attribute
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_isDestroyOwnedObjects(boolean v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setFlagCmd, v ? (byte)1 : (byte)0);
		UmlCom.check();
		_owned_objects = v;
	}
}
