package org.bouml.plugout.apibase.umlitem;

import org.bouml.plugout.apibase.PlugoutException;
import org.bouml.plugout.apibase.com.OnInstanceCmd;
import org.bouml.plugout.apibase.com.UmlCom;
import org.bouml.plugout.apibase.type.anExpansionKind;
import org.bouml.plugout.apibase.type.anItemKind;

public class UmlExpansionRegion extends UmlActivityRegion {
	/**
	 * returns a new expansion region named 's' created under 'parent'
	 * In case it cannot be created (the name is already used or invalid,
	 * 'parent' cannot contain it etc ...) return 0 in C++ and produce a
	 * PlugoutException in Java
	 */
	public static UmlExpansionRegion create(UmlItem parent, String s) throws PlugoutException {
		return (UmlExpansionRegion)parent.create_(anItemKind.anExpansionRegion, s);
	}

	private anExpansionKind _mode;

	private boolean _must_isolate;

	/**
	 * the constructor, do not call it yourself !!!!!!!!!!
	 */
	protected UmlExpansionRegion(long id, String s) {
		super(id, s);
	}

	/**
	 * return the isMustIsolate attribute, if TRUE the actions in the region
	 * execute in isolation from actions outside the region.
	 */
	public boolean isMustIsolate() {
		read_if_needed_();
		return _must_isolate;
	}

	@Override
	public anItemKind kind() {
		return anItemKind.anExpansionRegion;
	}

	/**
	 * return the mode attribute, the way in which the execution interact.
	 */
	public anExpansionKind mode() {
		read_if_needed_();
		return _mode;
	}

	@Override
	protected void read_uml_() {
		super.read_uml_();
		_must_isolate = UmlCom.read_bool();
		_mode = anExpansionKind.fromInt(UmlCom.read_char());
	}

	/**
	 * set the isMustIsolate attribute
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_isMustIsolate(boolean v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setFlagCmd, v ? (byte)1 : (byte)0);
		UmlCom.check();
		_must_isolate = v;
	}

	/**
	 * set the mode attribute
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_Mode(anExpansionKind v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setDefCmd, (byte)v.value());
		UmlCom.check();
		_mode = v;
	}
}
