package org.bouml.plugout.apibase.umlitem;

import org.bouml.plugout.apibase.FlowBehavior;
import org.bouml.plugout.apibase.PlugoutException;
import org.bouml.plugout.apibase.com.OnInstanceCmd;
import org.bouml.plugout.apibase.com.UmlCom;
import org.bouml.plugout.apibase.type.anItemKind;

public class UmlFlow extends UmlItem {
	/**
	 * Returns a new flow from 'start' to 'end'
	 * In case it cannot be created ('parent' cannot contain it etc ...) return
	 * 0 in C++ and produce a PlugoutException in Java
	 */
	public static UmlFlow create(UmlActivityNode start, UmlActivityNode end) throws PlugoutException {
		UmlCom.send_cmd(start.identifier_(), OnInstanceCmd.createCmd, anItemKind.aFlow, end.identifier_());
		final UmlFlow result = (UmlFlow)UmlItem.read_();
		if(result != null) {
			start.reread_children_if_needed_();
		} else {
			throw new PlugoutException("Cannot create the flow");
		}
		return result;
	}

	private FlowBehavior _cpp;

	private FlowBehavior _java;

	private UmlActivityNode _target;

	private FlowBehavior _uml;

	/**
	 * the constructor, do not call it yourself !!!!!!!!!!
	 */
	protected UmlFlow(long id, String s) {
		super(id, s);
	}

	/**
	 * return the guard in C++
	 */
	public String cppGuard() {
		read_if_needed_();
		return _cpp.guard;
	}

	/**
	 * return the selection in C++
	 */
	public String cppSelection() {
		read_if_needed_();
		return _cpp.selection;
	}

	/**
	 * return the transformation in C++
	 */
	public String cppTransformation() {
		read_if_needed_();
		return _cpp.transformation;
	}

	/**
	 * return the weight in C++
	 */
	public String cppWeight() {
		read_if_needed_();
		return _cpp.weight;
	}

	/**
	 * return the guard in OCL
	 */
	public String guard() {
		read_if_needed_();
		return _uml.guard;
	}

	/**
	 * return the guard in Java
	 */
	public String javaGuard() {
		read_if_needed_();
		return _java.guard;
	}

	/**
	 * return the selection in Java
	 */
	public String javaSelection() {
		read_if_needed_();
		return _java.selection;
	}

	/**
	 * return the transformation in Java
	 */
	public String javaTransformation() {
		read_if_needed_();
		return _java.transformation;
	}

	/**
	 * return the weight in Java
	 */
	public String javaWeight() {
		read_if_needed_();
		return _java.weight;
	}

	@Override
	public anItemKind kind() {
		return anItemKind.aFlow;
	}

	@Override
	protected void read_cpp_() {
		_cpp = new FlowBehavior();
		_cpp.read();
	}

	@Override
	protected void read_java_() {
		_java = new FlowBehavior();
		_java.read();
	}

	@Override
	protected void read_uml_() {
		super.read_uml_();
		_target = (UmlActivityNode)UmlItem.read_();
		_uml = new FlowBehavior();
		_uml.read();
	}

	/**
	 * return the selection in OCL
	 */
	public String selection() {
		read_if_needed_();
		return _uml.selection;
	}

	/**
	 * set the guard in C++
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_CppGuard(String v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setCppGuardCmd, v);
		UmlCom.check();
		_cpp.guard = v;
	}

	/**
	 * set the selection in C++
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_CppSelection(String v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setCppEntryBehaviorCmd, v);
		UmlCom.check();
		_cpp.selection = v;
	}

	/**
	 * set the transformation in C++
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_CppTransformation(String v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setCppExitBehaviorCmd, v);
		UmlCom.check();
		_cpp.transformation = v;
	}

	/**
	 * set the weight in C++
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_CppWeight(String v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setCppActivityCmd, v);
		UmlCom.check();
		_cpp.weight = v;
	}

	/**
	 * set the guard in OCL
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_Guard(String v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setUmlGuardCmd, v);
		UmlCom.check();
		_uml.guard = v;
	}

	/**
	 * set the guard in Java
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_JavaGuard(String v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setJavaGuardCmd, v);
		UmlCom.check();
		_java.guard = v;
	}

	/**
	 * set the selection in Java
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_JavaSelection(String v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setJavaEntryBehaviorCmd, v);
		UmlCom.check();
		_java.selection = v;
	}

	/**
	 * set the transformation in Java
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_JavaTransformation(String v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setJavaExitBehaviorCmd, v);
		UmlCom.check();
		_java.transformation = v;
	}

	/**
	 * set the weight in Java
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_JavaWeight(String v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setJavaActivityCmd, v);
		UmlCom.check();
		_java.weight = v;
	}

	/**
	 * set the selection in OCL
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_Selection(String v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setUmlEntryBehaviorCmd, v);
		UmlCom.check();
		_uml.selection = v;
	}

	/**
	 * set the transformation in OCL
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_Transformation(String v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setUmlExitBehaviorCmd, v);
		UmlCom.check();
		_uml.transformation = v;
	}

	/**
	 * set the weight in OCL
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_Weight(String v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setUmlActivityCmd, v);
		UmlCom.check();
		_uml.weight = v;
	}

	/**
	 * returns the 'end' object (the 'start' object is the parent of the flow)
	 * no set !
	 */
	public UmlActivityNode target() {
		read_if_needed_();
		return _target;
	}

	/**
	 * return the transformation in OCL
	 */
	public String transformation() {
		read_if_needed_();
		return _uml.transformation;
	}

	@Override
	public void unload(boolean rec, boolean del) {
		_uml = null;
		_cpp = null;
		_java = null;
		super.unload(rec, del);
	}

	/**
	 * return the weight in OCL
	 */
	public String weight() {
		read_if_needed_();
		return _uml.weight;
	}
}
