package org.bouml.plugout.apibase.umlitem;

import java.util.HashMap;
import org.bouml.plugout.apibase.PlugoutException;
import org.bouml.plugout.apibase.com.UmlCom;
import org.bouml.plugout.apibase.type.anItemKind;

public final class UmlItemFactory {
	private static UmlItemFactory singleton = null;

	public static UmlItemFactory getSingleton() {
		return singleton == null ? singleton = new UmlItemFactory() : singleton;
	}

	private final java.util.Map<Integer, Class<? extends UmlItem>> items;

	private UmlItemFactory() {
		items = new HashMap<Integer, Class<? extends UmlItem>>();
	}

	private final UmlItem getBaseInstance(int itemKind, long id, String name) {
		switch(itemKind) {
			case anItemKind._aRelation:
				return new UmlRelation(id, name);
			case anItemKind._anAttribute:
				return new UmlAttribute(id, name);
			case anItemKind._anOperation:
				return new UmlOperation(id, name);
			case anItemKind._anExtraClassMember:
				return new UmlExtraClassMember(id, name);
			case anItemKind._aClass:
				return new UmlClass(id, name);
			case anItemKind._anUseCase:
				return new UmlUseCase(id, name);
			case anItemKind._aComponent:
				return new UmlComponent(id, name);
			case anItemKind._anArtifact:
				return new UmlArtifact(id, name);
			case anItemKind._aNode:
				return new UmlNode(id, name);
			case anItemKind._aNcRelation:
				return new UmlNcRelation(id, name);
			case anItemKind._aClassDiagram:
				return new UmlClassDiagram(id, name);
			case anItemKind._anUseCaseDiagram:
				return new UmlUseCaseDiagram(id, name);
			case anItemKind._aSequenceDiagram:
				return new UmlSequenceDiagram(id, name);
			case anItemKind._aCollaborationDiagram:
				return new UmlCollaborationDiagram(id, name);
			case anItemKind._aComponentDiagram:
				return new UmlComponentDiagram(id, name);
			case anItemKind._aDeploymentDiagram:
				return new UmlDeploymentDiagram(id, name);
			case anItemKind._aClassView:
				return new UmlClassView(id, name);
			case anItemKind._anUseCaseView:
				return new UmlUseCaseView(id, name);
			case anItemKind._aComponentView:
				return new UmlComponentView(id, name);
			case anItemKind._aDeploymentView:
				return new UmlDeploymentView(id, name);
			case anItemKind._aPackage:
				return new UmlPackage(id, name);
			case anItemKind._aState:
				return new UmlState(id, name);
			case anItemKind._aTransition:
				return new UmlTransition(id, name);
			case anItemKind._aRegion:
				return new UmlRegion(id, name);
			case anItemKind._aStateDiagram:
				return new UmlStateDiagram(id, name);
			case anItemKind._aStateAction:
				return new UmlStateAction(id, "");
			case anItemKind._anInitialPseudoState:
				return new UmlInitialPseudoState(id, "");
			case anItemKind._anEntryPointPseudoState:
				return new UmlEntryPointPseudoState(id, "");
			case anItemKind._aFinalState:
				return new UmlFinalState(id, "");
			case anItemKind._aTerminatePseudoState:
				return new UmlTerminatePseudoState(id, "");
			case anItemKind._anExitPointPseudoState:
				return new UmlExitPointPseudoState(id, name);
			case anItemKind._aDeepHistoryPseudoState:
				return new UmlDeepHistoryPseudoState(id, "");
			case anItemKind._aShallowHistoryPseudoState:
				return new UmlShallowHistoryPseudoState(id, "");
			case anItemKind._aJunctionPseudoState:
				return new UmlJunctionPseudoState(id, "");
			case anItemKind._aChoicePseudoState:
				return new UmlChoicePseudoState(id, "");
			case anItemKind._aForkPseudoState:
				return new UmlForkPseudoState(id, "");
			case anItemKind._aJoinPseudoState:
				return new UmlJoinPseudoState(id, "");
			case anItemKind._anObjectDiagram:
				return new UmlObjectDiagram(id, name);
			case anItemKind._anActivityDiagram:
				return new UmlActivityDiagram(id, name);
			case anItemKind._anActivity:
				return new UmlActivity(id, name);
			case anItemKind._aFlow:
				return new UmlFlow(id, name);
			case anItemKind._anActivityParameter:
				return new UmlActivityParameter(id, name);
			case anItemKind._aParameterSet:
				return new UmlParameterSet(id, name);
			case anItemKind._anExpansionRegion:
				return new UmlExpansionRegion(id, name);
			case anItemKind._anInterruptibleActivityRegion:
				return new UmlInterruptibleActivityRegion(id, name);
			case anItemKind._anOpaqueAction:
				return new UmlOpaqueAction(id, name);
			case anItemKind._anAcceptEventAction:
				return new UmlAcceptEventAction(id, name);
			case anItemKind._aReadVariableValueAction:
				return new UmlReadVariableValueAction(id, name);
			case anItemKind._aClearVariableValueAction:
				return new UmlClearVariableValueAction(id, name);
			case anItemKind._aWriteVariableValueAction:
				return new UmlWriteVariableValueAction(id, name);
			case anItemKind._anAddVariableValueAction:
				return new UmlAddVariableValueAction(id, name);
			case anItemKind._aRemoveVariableValueAction:
				return new UmlRemoveVariableValueAction(id, name);
			case anItemKind._aCallBehaviorAction:
				return new UmlCallBehaviorAction(id, name);
			case anItemKind._aCallOperationAction:
				return new UmlCallOperationAction(id, name);
			case anItemKind._aSendObjectAction:
				return new UmlSendObjectAction(id, name);
			case anItemKind._aSendSignalAction:
				return new UmlSendSignalAction(id, name);
			case anItemKind._aBroadcastSignalAction:
				return new UmlBroadcastSignalAction(id, name);
			case anItemKind._anUnmarshallAction:
				return new UmlUnmarshallAction(id, name);
			case anItemKind._aValueSpecificationAction:
				return new UmlValueSpecificationAction(id, name);
			case anItemKind._anActivityObject:
				return new UmlActivityObject(id, name);
			case anItemKind._anExpansionNode:
				return new UmlExpansionNode(id, name);
			case anItemKind._anActivityPin:
				return new UmlActivityPin(id, name);
			case anItemKind._anInitialActivityNode:
				return new UmlInitialActivityNode(id, name);
			case anItemKind._aFlowFinalActivityNode:
				return new UmlFlowFinalActivityNode(id, name);
			case anItemKind._anActivityFinalActivityNode:
				return new UmlActivityFinalActivityNode(id, name);
			case anItemKind._aDecisionActivityNode:
				return new UmlDecisionActivityNode(id, name);
			case anItemKind._aMergeActivityNode:
				return new UmlMergeActivityNode(id, name);
			case anItemKind._aForkActivityNode:
				return new UmlForkActivityNode(id, name);
			case anItemKind._aJoinActivityNode:
				return new UmlJoinActivityNode(id, name);
			case anItemKind._aPartition:
				return new UmlActivityPartition(id, name);
			case anItemKind._aClassInstance:
				return new UmlClassInstance(id, name);
			case anItemKind._anAcceptCallAction:
				return new UmlAcceptCallAction(id, name);
			case anItemKind._aReplyAction:
				return new UmlReplyAction(id, name);
			case anItemKind._aCreateObjectAction:
				return new UmlCreateObjectAction(id, name);
			case anItemKind._aDestroyObjectAction:
				return new UmlDestroyObjectAction(id, name);
			case anItemKind._aTestIdentityAction:
				return new UmlTestIdentityAction(id, name);
			case anItemKind._aRaiseExceptionAction:
				return new UmlRaiseExceptionAction(id, name);
			case anItemKind._aReduceAction:
				return new UmlReduceAction(id, name);
			default:
				throw new PlugoutException("Internal error, uml kind  '" + itemKind + " 'unknow.");
		}
	}

	public UmlItem getInstance(int itemKind, long id, String name) {
		if(!items.containsKey(itemKind)) {
			return getBaseInstance(itemKind, id, name);
		} else {
			try {
				return items.get(itemKind).getConstructor(Long.TYPE, String.class).newInstance(id, name);
			} catch(final Exception exception) {
				UmlCom.traceException(exception);
				return null;
			}
		}
	}

	public void setItems(anItemKind itemKind, Class<? extends UmlItem> umlItem) {
		items.put(itemKind.value(), umlItem);
	}
}
