package org.bouml.plugout.apibase.umlitem;

import org.bouml.plugout.apibase.PlugoutException;
import org.bouml.plugout.apibase.com.OnInstanceCmd;
import org.bouml.plugout.apibase.com.UmlCom;
import org.bouml.plugout.apibase.type.aRelationKind;
import org.bouml.plugout.apibase.type.anItemKind;

/**
 * Manage the relations between non class objects, for instances dependency
 * between packages and use case generalization
 */
public class UmlNcRelation extends UmlItem {
	/**
	 * returns a new non class relation of the given 'kind' from 'start' to
	 * 'end'
	 * In case it cannot be created ('parent' cannot contain it etc ...) return
	 * 0 in C++ and produce a PlugoutException in Java
	 */
	public static UmlNcRelation create(aRelationKind kind, UmlItem start, UmlItem end) throws PlugoutException {
		UmlCom.send_cmd(start.identifier_(), OnInstanceCmd.createCmd, anItemKind.aNcRelation, kind, end.identifier_());
		final UmlNcRelation result = (UmlNcRelation)UmlItem.read_();
		if(result != null) {
			start.reread_children_if_needed_();
		} else {
			throw new PlugoutException("can't be created");
		}
		return result;
	}

	private aRelationKind _rel_kind;

	private UmlItem _target;

	/**
	 * the constructor, do not call it yourself !!!!!!!!!!
	 */
	protected UmlNcRelation(long id, String n) {
		super(id, n);
	}

	@Override
	public anItemKind kind() {
		return anItemKind.aNcRelation;
	}

	@Override
	protected void read_uml_() {
		super.read_uml_();
		_rel_kind = aRelationKind.fromInt(UmlCom.read_char());
		_target = UmlItem.read_();
	}

	/**
	 * returns the kind of the relation : aGeneralisation or a Dependency, the
	 * other are not allowed for the non class relations
	 */
	public aRelationKind relationKind() {
		read_if_needed_();
		return _rel_kind;
	}

	/**
	 * returns the 'end' object (the 'start' object is the parent of the
	 * relation) no set !
	 */
	public UmlItem target() {
		read_if_needed_();
		return _target;
	}
}
