package org.bouml.plugout.apibase.umlitem;

import org.bouml.plugout.apibase.PlugoutException;
import org.bouml.plugout.apibase.com.OnInstanceCmd;
import org.bouml.plugout.apibase.com.UmlCom;
import org.bouml.plugout.apibase.type.anItemKind;

/**
 * Manage the nodes
 */
public class UmlNode extends UmlItem {
	/**
	 * returns a new node named 's' created under 'parent'
	 * In case it cannot be created (the name is already used or invalid,
	 * 'parent' cannot contain it etc ...) return 0 in C++ and produce a
	 * PlugoutException in Java
	 */
	public static UmlNode create(UmlItem parent, String s) throws PlugoutException {
		return (UmlNode)parent.create_(anItemKind.aNode, s);
	}

	private UmlDeploymentDiagram _assoc_diagram;

	/**
	 * the constructor, do not call it yourself !!!!!!!!!!
	 */
	protected UmlNode(long id, String n) {
		super(id, n);
	}

	/**
	 * returns the optional associated diagram
	 */
	public UmlDeploymentDiagram associatedDiagram() {
		read_if_needed_();
		return _assoc_diagram;
	}

	@Override
	public anItemKind kind() {
		return anItemKind.aNode;
	}

	@Override
	protected void read_uml_() {
		_assoc_diagram = (UmlDeploymentDiagram)UmlItem.read_();
		super.read_uml_();
	}

	/**
	 * sets the associated diagram, arg may be null to unset it
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_AssociatedDiagram(UmlDeploymentDiagram d) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setAssocDiagramCmd, d == null ? 0 : d == null ? (long)0 : d.identifier_());
		UmlCom.check();
		_assoc_diagram = d;
	}
}
