package org.bouml.plugout.apibase.umlitem;

import org.bouml.plugout.apibase.PlugoutException;
import org.bouml.plugout.apibase.com.OnInstanceCmd;
import org.bouml.plugout.apibase.com.UmlCom;

public abstract class UmlOnSignalAction extends UmlActivityAction {
	private String _cpp_signal;

	private String _java_signal;

	private String _signal;

	/**
	 * the constructor, do not call it yourself !!!!!!!!!!
	 */
	protected UmlOnSignalAction(long id, String s) {
		super(id, s);
	}

	/**
	 * return the signal in C++
	 */
	public String cppSignal() {
		read_if_needed_();
		return _cpp_signal;
	}

	/**
	 * return the signal in Java
	 */
	public String javaSignal() {
		read_if_needed_();
		return _java_signal;
	}

	@Override
	protected void read_cpp_() {
		super.read_cpp_();
		_cpp_signal = UmlCom.read_string();
	}

	@Override
	protected void read_java_() {
		super.read_java_();
		_java_signal = UmlCom.read_string();
	}

	@Override
	protected void read_uml_() {
		super.read_uml_();
		_signal = UmlCom.read_string();
	}

	/**
	 * set the signal in C++
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_CppSignal(String v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setCppActivityCmd, v);
		UmlCom.check();
		_cpp_signal = v;
	}

	/**
	 * set the signal in Java
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_JavaSignal(String v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setJavaActivityCmd, v);
		UmlCom.check();
		_java_signal = v;
	}

	/**
	 * set the signal
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_Signal(String v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setUmlActivityCmd, v);
		UmlCom.check();
		_signal = v;
	}

	/**
	 * return the signal
	 */
	public String signal() {
		read_if_needed_();
		return _signal;
	}

	/**
	 * to unload the object to free memory, it will be reloaded automatically if
	 * needed. Recursively done for the sub items if 'rec' is TRUE.
	 * if 'del' is true the sub items are deleted in C++, and removed from the
	 * internal dictionnary in C++ and Java (to allow it to be garbaged), you
	 * will have to call Children() to re-access to them
	 */
	@Override
	public void unload(boolean rec, boolean del) {
		_signal = null;
		_cpp_signal = null;
		_java_signal = null;
		super.unload(rec, del);
	}
}
