package org.bouml.plugout.apibase.umlitem;

import org.bouml.plugout.apibase.PlugoutException;
import org.bouml.plugout.apibase.com.OnInstanceCmd;
import org.bouml.plugout.apibase.com.UmlCom;
import org.bouml.plugout.apibase.type.anItemKind;

public class UmlOpaqueAction extends UmlActivityAction {
	/**
	 * returns a new opaque action named 's' created under 'parent'
	 * In case it cannot be created (the name is already used or invalid,
	 * 'parent' cannot contain it etc ...) return 0 in C++ and produce a
	 * PlugoutException in Java
	 */
	public static UmlOpaqueAction create(UmlItem parent, String s) throws PlugoutException {
		return (UmlOpaqueAction)parent.create_(anItemKind.anOpaqueAction, s);
	}

	private String _behavior;

	private String _cpp_behavior;

	private String _java_behavior;

	/**
	 * the constructor, do not call it yourself !!!!!!!!!!
	 */
	protected UmlOpaqueAction(long id, String s) {
		super(id, s);
	}

	/**
	 * return the behavior
	 */
	public String behavior() {
		read_if_needed_();
		return _behavior;
	}

	/**
	 * return the behavior in C++
	 */
	public String cppBehavior() {
		read_if_needed_();
		return _cpp_behavior;
	}

	/**
	 * return the behavior in Java
	 */
	public String javaBehavior() {
		read_if_needed_();
		return _java_behavior;
	}

	@Override
	public anItemKind kind() {
		return anItemKind.anOpaqueAction;
	}

	@Override
	protected void read_cpp_() {
		super.read_cpp_();
		_cpp_behavior = UmlCom.read_string();
	}

	@Override
	protected void read_java_() {
		super.read_java_();
		_java_behavior = UmlCom.read_string();
	}

	@Override
	protected void read_uml_() {
		super.read_uml_();
		_behavior = UmlCom.read_string();
	}

	/**
	 * set the behavior
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_Behavior(String v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setUmlActivityCmd, v);
		UmlCom.check();
		_behavior = v;
	}

	/**
	 * set the behavior in C++
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_CppBehavior(String v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setCppActivityCmd, v);
		UmlCom.check();
		_cpp_behavior = v;
	}

	/**
	 * set the behavior in Java
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_JavaBehavior(String v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setJavaActivityCmd, v);
		UmlCom.check();
		_java_behavior = v;
	}

	@Override
	public void unload(boolean rec, boolean del) {
		_behavior = null;
		_cpp_behavior = null;
		_java_behavior = null;
		super.unload(rec, del);
	}
}
