package org.bouml.plugout.apibase.umlitem;

import org.bouml.plugout.apibase.PlugoutException;
import org.bouml.plugout.apibase.com.OnInstanceCmd;
import org.bouml.plugout.apibase.com.UmlCom;
import org.bouml.plugout.apibase.type.anItemKind;

public class UmlParameterSet extends UmlItem {
	/**
	 * returns a new parameter set named 's' created under 'parent'
	 * In case it cannot be created (the name is already used or invalid,
	 * 'parent' cannot contain it etc ...) return 0 in C++ and produce a
	 * PlugoutException in Java
	 */
	public static UmlActivityParameter create(UmlActivityAction parent, String s) throws PlugoutException {
		return (UmlActivityParameter)parent.create_(anItemKind.aParameterSet, s);
	}

	private UmlActivityPin[] _pins;

	/**
	 * the constructor, do not call it yourself !!!!!!!!!!
	 */
	protected UmlParameterSet(long id, String s) {
		super(id, s);
	}

	@Override
	public anItemKind kind() {
		return anItemKind.aParameterSet;
	}

	/**
	 * return the pins part of the parameter set
	 */
	public UmlActivityPin[] pins() {
		read_if_needed_();
		return _pins.clone();
	}

	@Override
	protected void read_uml_() {
		super.read_uml_();
		final int n = UmlCom.read_unsigned();
		_pins = new UmlActivityPin[n];
		for(int index = 0 ; index != n ; index += 1) {
			_pins[index] = (UmlActivityPin)UmlItem.read_();
		}
	}

	/**
	 * set the pins
	 */
	public void set_Pins(UmlActivityPin[] v) {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.replaceParameterCmd, v);
		UmlCom.check();
		_pins = v.clone();
	}

	@Override
	public void unload(boolean rec, boolean del) {
		_pins = null;
		super.unload(rec, del);
	}
}
