package org.bouml.plugout.apibase.umlitem;

import org.bouml.plugout.apibase.PlugoutException;
import org.bouml.plugout.apibase.com.OnInstanceCmd;
import org.bouml.plugout.apibase.com.UmlCom;
import org.bouml.plugout.apibase.type.aDirection;
import org.bouml.plugout.apibase.type.aParameterEffectKind;

public abstract class UmlPinParameter extends UmlActivityObject {
	private aDirection _dir;

	private aParameterEffectKind _effect;

	private boolean _exception;

	private boolean _stream;

	private boolean _unique;

	/**
	 * the constructor, do not call it yourself !!!!!!!!!!
	 */
	protected UmlPinParameter(long id, String s) {
		super(id, s);
	}

	/**
	 * return the direction
	 */
	public aDirection direction() {
		read_if_needed_();
		return _dir;
	}

	/**
	 * return the effect
	 */
	public aParameterEffectKind effect() {
		read_if_needed_();
		return _effect;
	}

	/**
	 * return the isException attribute, exclusive with isStream
	 */
	public boolean isException() {
		read_if_needed_();
		return _exception;
	}

	/**
	 * return the isStream attribute
	 */
	public boolean isStream() {
		read_if_needed_();
		return _stream;
	}

	/**
	 * return the isUnique attribute
	 */
	public boolean isUnique() {
		read_if_needed_();
		return _unique;
	}

	@Override
	protected void read_uml_() {
		super.read_uml_();
		_unique = UmlCom.read_bool();
		_exception = UmlCom.read_bool();
		_stream = UmlCom.read_bool();
		_dir = aDirection.fromInt(UmlCom.read_char());
		_effect = aParameterEffectKind.fromInt(UmlCom.read_char());
	}

	/**
	 * set the direction
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_Direction(aDirection v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setIdlDirCmd, (byte)v.value());
		UmlCom.check();
		_dir = v;
	}

	/**
	 * set the effect
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_Effect(aParameterEffectKind v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.replaceParameterCmd, (byte)v.value());
		UmlCom.check();
		_effect = v;
	}

	/**
	 * set the isException attribute, exclusive with isStream
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_IsException(boolean v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.replaceExceptionCmd, v ? 1 : 0);
		UmlCom.check();
		_exception = v;
	}

	/**
	 * set the isStream attribute
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_IsStream(boolean v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setStreamCmd, v ? 1 : 0);
		UmlCom.check();
		_stream = v;
	}

	/**
	 * set the isUnique attribute
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_IsUnique(boolean v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setUniqueCmd, v ? 1 : 0);
		UmlCom.check();
		_unique = v;
	}
}
