package org.bouml.plugout.apibase.umlitem;

import org.bouml.plugout.apibase.PlugoutException;
import org.bouml.plugout.apibase.com.OnInstanceCmd;
import org.bouml.plugout.apibase.com.UmlCom;
import org.bouml.plugout.apibase.type.anItemKind;

public class UmlReduceAction extends UmlActivityAction {
	/**
	 * returns a new reduce action named 's' created under 'parent'
	 * In case it cannot be created (the name is already used or invalid,
	 * 'parent' cannot contain it etc ...) return 0 in C++ and produce a
	 * PlugoutException in Java
	 */
	public static UmlReduceAction create(UmlItem parent, String s) throws PlugoutException {
		return (UmlReduceAction)parent.create_(anItemKind.aReduceAction, s);
	}

	private boolean _ordered;

	private UmlItem _reducer;

	/**
	 * the constructor, do not call it yourself !!!!!!!!!!
	 */
	protected UmlReduceAction(long id, String s) {
		super(id, s);
	}

	/**
	 * return the return the isOrdered attribute
	 */
	public boolean isOrdered() {
		read_if_needed_();
		return _ordered;
	}

	@Override
	public anItemKind kind() {
		return anItemKind.aReduceAction;
	}

	@Override
	protected void read_uml_() {
		super.read_uml_();
		_ordered = UmlCom.read_bool();
		_reducer = UmlItem.read_();
	}

	/**
	 * return the reducer, may be an activity or a state machine
	 */
	public UmlItem reducer() {
		read_if_needed_();
		return _reducer;
	}

	/**
	 * set the isOrdered attribute
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_isOrdered(boolean v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setFlagCmd, v ? (byte)1 : (byte)0);
		UmlCom.check();
		_ordered = v;
	}

	/**
	 * set the reducer
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_Reducer(UmlItem v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setDefCmd, v == null ? (long)0 : v.identifier_());
		UmlCom.check();
		_reducer = v;
	}
}
