package org.bouml.plugout.apibase.umlitem;

import org.bouml.plugout.apibase.PlugoutException;
import org.bouml.plugout.apibase.com.OnInstanceCmd;
import org.bouml.plugout.apibase.com.UmlCom;
import org.bouml.plugout.apibase.type.anItemKind;

public class UmlReplyAction extends UmlActivityAction {
	/**
	 * returns a new reply action named 's' created under 'parent'
	 * In case it cannot be created (the name is already used or invalid,
	 * 'parent' cannot contain it etc ...) return 0 in C++ and produce a
	 * PlugoutException in Java
	 */
	public static UmlReplyAction create(UmlItem parent, String s) throws PlugoutException {
		return (UmlReplyAction)parent.create_(anItemKind.aReplyAction, s);
	}

	private String _cpp_trigger;

	private String _java_trigger;

	private String _trigger;

	/**
	 * the constructor, do not call it yourself !!!!!!!!!!
	 */
	protected UmlReplyAction(long id, String s) {
		super(id, s);
	}

	/**
	 * return the replyToCall trigger in C++
	 */
	public String cppReplyToCall() {
		read_if_needed_();
		return _cpp_trigger;
	}

	/**
	 * return the replyToCall trigger in Java
	 */
	public String javaReplyToCall() {
		read_if_needed_();
		return _java_trigger;
	}

	@Override
	public anItemKind kind() {
		return anItemKind.aReplyAction;
	}

	@Override
	protected void read_cpp_() {
		super.read_cpp_();
		_cpp_trigger = UmlCom.read_string();
	}

	@Override
	protected void read_java_() {
		super.read_java_();
		_java_trigger = UmlCom.read_string();
	}

	@Override
	protected void read_uml_() {
		super.read_uml_();
		_trigger = UmlCom.read_string();
	}

	/**
	 * return the replyToCall trigger
	 */
	public String replyToCall() {
		read_if_needed_();
		return _trigger;
	}

	/**
	 * set the replyToCall trigger in C++
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_CppReplyToCall(String v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setCppTriggerCmd, v);
		UmlCom.check();
		_cpp_trigger = v;
	}

	/**
	 * set the replyToCall trigger in Java
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_JavaReplyToCall(String v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setJavaTriggerCmd, v);
		UmlCom.check();
		_java_trigger = v;
	}

	/**
	 * set the replyToCall trigger
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_ReplyToCall(String v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setUmlTriggerCmd, v);
		UmlCom.check();
		_trigger = v;
	}

	@Override
	public void unload(boolean rec, boolean del) {
		_trigger = null;
		_cpp_trigger = null;
		_java_trigger = null;
		super.unload(rec, del);
	}
}
