package org.bouml.plugout.apibase.umlitem;

import org.bouml.plugout.apibase.PlugoutException;
import org.bouml.plugout.apibase.com.OnInstanceCmd;
import org.bouml.plugout.apibase.com.UmlCom;
import org.bouml.plugout.apibase.diagram.UmlSequenceDiagramDefinition;
import org.bouml.plugout.apibase.type.anItemKind;

/**
 * Manage the sequence diagrams
 */
public class UmlSequenceDiagram extends UmlDiagram {
	/**
	 * returns a new sequence diagram named 's' created under 'parent'
	 * In case it cannot be created (the name is already used or invalid,
	 * 'parent' cannot contain it etc ...) return 0 in C++ and produce a
	 * PlugoutException in Java
	 */
	public static UmlSequenceDiagram create(UmlItem parent, String s) throws PlugoutException {
		return (UmlSequenceDiagram)parent.create_(anItemKind.aSequenceDiagram, s);
	}

	private UmlSequenceDiagramDefinition _def;

	/**
	 * the constructor, do not call it yourself !!!!!!!!!!
	 */
	protected UmlSequenceDiagram(long id, String n) {
		super(id, n);
		_def = null;
	}

	/**
	 * return the semantic part of the diagram not present in the model
	 */
	public UmlSequenceDiagramDefinition definition() {
		if(_def == null) {
			UmlCom.send_cmd(identifier_(), OnInstanceCmd.sideCmd);
			(_def = new UmlSequenceDiagramDefinition()).read_();
		}
		return _def;
	}

	@Override
	public anItemKind kind() {
		return anItemKind.aSequenceDiagram;
	}

	@Override
	public void unload(boolean rec, boolean del) {
		_def = null;
		super.unload(rec, del);
	}
}
