package org.bouml.plugout.apibase.umlitem;

import org.bouml.plugout.apibase.PlugoutException;
import org.bouml.plugout.apibase.StateBehavior;
import org.bouml.plugout.apibase.com.OnInstanceCmd;
import org.bouml.plugout.apibase.com.UmlCom;
import org.bouml.plugout.apibase.type.anItemKind;

public class UmlState extends UmlItem {
	/**
	 * returns a new state named 's' created under 'parent'
	 * In case it cannot be created (the name is already used or invalid,
	 * 'parent' cannot contain it etc ...) return 0 in C++ and produce a
	 * RuntimeException in Java
	 */
	public static UmlState create(UmlItem parent, String s) throws RuntimeException {
		return (UmlState)parent.create_(anItemKind.aState, s);
	}

	private boolean _active;

	private UmlStateDiagram _assoc_diagram;

	private StateBehavior _cpp;

	private StateBehavior _java;

	private UmlOperation _specification;

	private StateBehavior _uml;

	/**
	 * The constructor, do not call it yourself !
	 */
	public UmlState(long id, String s) {
		super(id, s);
	}

	/**
	 * Returns the optional associated diagram.
	 */
	public UmlStateDiagram associatedDiagram() {
		read_if_needed_();
		return _assoc_diagram;
	}

	/**
	 * Return the activity in C++.
	 */
	public String cppDoActivity() {
		read_if_needed_();
		return _cpp.do_activity;
	}

	/**
	 * Return the entry behavior in C++.
	 */
	public String cppEntryBehavior() {
		read_if_needed_();
		return _cpp.on_entry;
	}

	/**
	 * Return the exit behavior in C++.
	 */
	public String cppExitBehavior() {
		read_if_needed_();
		return _cpp.on_exit;
	}

	/**
	 * Return the activity in OCL.
	 */
	public String doActivity() {
		read_if_needed_();
		return _uml.do_activity;
	}

	/**
	 * Return the entry behavior in OCL.
	 */
	public String entryBehavior() {
		read_if_needed_();
		return _uml.on_entry;
	}

	/**
	 * Return the exit behavior in OCL.
	 */
	public String exitBehavior() {
		read_if_needed_();
		return _uml.on_exit;
	}

	/**
	 * Return the indicate if the class is active.
	 */
	public boolean isActive() {
		read_if_needed_();
		return _active;
	}

	/**
	 * Return the activity in Java.
	 */
	public String javaDoActivity() {
		read_if_needed_();
		return _java.do_activity;
	}

	/**
	 * Return the entry behavior in Java.
	 */
	public String javaEntryBehavior() {
		read_if_needed_();
		return _java.on_entry;
	}

	/**
	 * Return the exit behavior in Java.
	 */
	public String javaExitBehavior() {
		read_if_needed_();
		return _java.on_exit;
	}

	@Override
	public anItemKind kind() {
		return anItemKind.aState;
	}

	@Override
	protected void read_cpp_() {
		_cpp = new StateBehavior();
		_cpp.read();
	}

	@Override
	protected void read_java_() {
		_java = new StateBehavior();
		_java.read();
	}

	@Override
	protected void read_uml_() {
		_assoc_diagram = (UmlStateDiagram)UmlItem.read_();
		super.read_uml_();
		_uml = new StateBehavior();
		_uml.read();
		_specification = (UmlOperation)UmlItem.read_();
		_active = UmlCom.read_bool();
	}

	/**
	 * Sets the associated diagram, arg may be null to unset it.
	 * On error produce a PlugoutException.
	 */
	public void set_AssociatedDiagram(UmlStateDiagram d) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setAssocDiagramCmd, d == null ? (long)0 : d.identifier_());
		UmlCom.check();
		_assoc_diagram = d;
	}

	/**
	 * Set the activity in C++.
	 * On error produce a PlugoutException.
	 */
	public void set_CppDoActivity(String s) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setCppActivityCmd, s);
		UmlCom.check();
		_cpp.do_activity = s;
	}

	/**
	 * Set the entry behavior in C++.
	 * On error produce a PlugoutException.
	 */
	public void set_CppEntryBehavior(String s) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setCppEntryBehaviorCmd, s);
		UmlCom.check();
		_cpp.on_entry = s;
	}

	/**
	 * Set the exit behavior in C++.
	 * On error produce a PlugoutException.
	 */
	public void set_CppExitBehavior(String s) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setCppExitBehaviorCmd, s);
		UmlCom.check();
		_cpp.on_exit = s;
	}

	/**
	 * Set the activity in OCL.
	 * On error produce a PlugoutException.
	 */
	public void set_DoActivity(String s) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setUmlActivityCmd, s);
		UmlCom.check();
		_uml.do_activity = s;
	}

	/**
	 * Set the entry behavior in OCL.
	 * On error produce a PlugoutException.
	 */
	public void set_EntryBehavior(String s) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setUmlEntryBehaviorCmd, s);
		UmlCom.check();
		_uml.on_entry = s;
	}

	/**
	 * Set the exit behavior in OCL.
	 * On error produce a PlugoutException.
	 */
	public void set_ExitBehavior(String s) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setUmlExitBehaviorCmd, s);
		UmlCom.check();
		_uml.on_exit = s;
	}

	/**
	 * Set if the class is active.
	 * On error produce a PlugoutException.
	 */
	public void set_isActive(boolean v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setActiveCmd, v ? 1 : 0);
		UmlCom.check();
		_active = v;
	}

	/**
	 * Set the activity in Java.
	 * On error produce a PlugoutException.
	 */
	public void set_JavaDoActivity(String s) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setJavaActivityCmd, s);
		UmlCom.check();
		_java.do_activity = s;
	}

	/**
	 * Set the entry behavior in Java.
	 * On error produce a PlugoutException.
	 */
	public void set_JavaEntryBehavior(String s) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setJavaEntryBehaviorCmd, s);
		UmlCom.check();
		_java.on_entry = s;
	}

	/**
	 * Set the exit behavior in Java.
	 * On error produce a PlugoutException.
	 */
	public void set_JavaExitBehavior(String s) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setJavaExitBehaviorCmd, s);
		UmlCom.check();
		_java.on_exit = s;
	}

	/**
	 * Set the specification.
	 * On error produce a PlugoutException.
	 */
	public void set_Specification(UmlOperation v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setDefCmd, v == null ? (long)0 : v.identifier_());
		UmlCom.check();
		_specification = v;
	}

	/**
	 * Return the specification.
	 */
	public UmlOperation specification() {
		read_if_needed_();
		return _specification;
	}

	@Override
	public void unload(boolean rec, boolean del) {
		_uml = null;
		_cpp = null;
		_java = null;
		super.unload(rec, del);
	}
}
