package org.bouml.plugout.apibase.umlitem;

import org.bouml.plugout.apibase.PlugoutException;
import org.bouml.plugout.apibase.com.OnInstanceCmd;
import org.bouml.plugout.apibase.com.UmlCom;
import org.bouml.plugout.apibase.type.anItemKind;

public class UmlStateAction extends UmlItem {
	/**
	 * returns a new state action created under 'parent'
	 * In case it cannot be created ('parent' cannot contain it etc ...) return
	 * 0 in C++ and produce a PlugoutException in Java
	 */
	public static UmlStateAction create(UmlItem parent) throws PlugoutException {
		return (UmlStateAction)parent.create_(anItemKind.aStateAction, "");
	}

	private String _cpp;

	private String _java;

	private String _uml;

	/**
	 * the constructor, do not call it yourself !!!!!!!!!!
	 */
	protected UmlStateAction(long id, String s) {
		super(id, s);
	}

	/**
	 * return the expression in C++
	 */
	public String cppExpression() {
		read_if_needed_();
		return _cpp;
	}

	/**
	 * return the expression in OCL
	 */
	public String expression() {
		read_if_needed_();
		return _uml;
	}

	/**
	 * return the expression in Java
	 */
	public String javaExpression() {
		read_if_needed_();
		return _java;
	}

	@Override
	public anItemKind kind() {
		return anItemKind.aStateAction;
	}

	@Override
	protected void read_cpp_() {
		_cpp = UmlCom.read_string();
	}

	@Override
	protected void read_java_() {
		_java = UmlCom.read_string();
	}

	@Override
	protected void read_uml_() {
		super.read_uml_();
		_uml = UmlCom.read_string();
	}

	/**
	 * set the expression in C++
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_CppExpression(String s) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setCppDeclCmd, s);
		UmlCom.check();
		_cpp = s;
	}

	/**
	 * set the expression in OCL
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_Expression(String s) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setDefCmd, s);
		UmlCom.check();
		_uml = s;
	}

	/**
	 * set the expression in Java
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_JavaExpression(String s) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setJavaDeclCmd, s);
		UmlCom.check();
		_java = s;
	}

	@Override
	public void unload(boolean rec, boolean del) {
		_uml = null;
		_cpp = null;
		_java = null;
		super.unload(rec, del);
	}
}
