package org.bouml.plugout.apibase.umlitem;

import org.bouml.plugout.apibase.PlugoutException;
import org.bouml.plugout.apibase.com.OnInstanceCmd;
import org.bouml.plugout.apibase.com.UmlCom;
import org.bouml.plugout.apibase.type.anItemKind;

/**
 * Manage the use cases
 */
public class UmlUseCase extends UmlItem {
	/**
	 * returns a new use case named 's' created under 'parent'
	 * In case it cannot be created (the name is already used or invalid,
	 * 'parent' cannot contain it etc ...) return 0 in C++ and produce a
	 * PlugoutException in Java
	 */
	public static UmlUseCase create(UmlItem parent, String s) throws PlugoutException {
		return (UmlUseCase)parent.create_(anItemKind.anUseCase, s);
	}

	private UmlUseCaseDiagram _assoc_diagram;

	private String _extension_points;

	/**
	 * the constructor, do not call it yourself !!!!!!!!!!
	 */
	protected UmlUseCase(long id, String n) {
		super(id, n);
	}

	/**
	 * returns the optional associated diagram
	 */
	public UmlUseCaseDiagram associatedDiagram() {
		read_if_needed_();
		return _assoc_diagram;
	}

	/**
	 * return the extension points
	 */
	public String extensionPoints() {
		read_if_needed_();
		return _extension_points;
	}

	@Override
	public anItemKind kind() {
		return anItemKind.anUseCase;
	}

	@Override
	protected void read_uml_() {
		_assoc_diagram = (UmlUseCaseDiagram)UmlItem.read_();
		super.read_uml_();
		_extension_points = UmlCom.read_string();
	}

	/**
	 * sets the associated diagram, arg may be null to unset it
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_AssociatedDiagram(UmlUseCaseDiagram d) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setAssocDiagramCmd, d == null ? (long)0 : d.identifier_());
		UmlCom.check();
		_assoc_diagram = d;
	}

	/**
	 * set the extension points
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_ExtensionPoints(String v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.replaceExceptionCmd, v);
		UmlCom.check();
		_extension_points = v;
	}
}
