package org.bouml.plugout.apibase.umlitem;

import org.bouml.plugout.apibase.PlugoutException;
import org.bouml.plugout.apibase.com.OnInstanceCmd;
import org.bouml.plugout.apibase.com.UmlCom;
import org.bouml.plugout.apibase.diagram.UmlUseCaseDiagramDefinition;
import org.bouml.plugout.apibase.type.anItemKind;

/**
 * Manage the use case diagrams
 */
public class UmlUseCaseDiagram extends UmlDiagram {
	/**
	 * returns a new use case diagram named 's' created under 'parent'
	 * In case it cannot be created (the name is already used or invalid,
	 * 'parent' cannot contain it etc ...) return 0 in C++ and produce a
	 * PlugoutException in Java
	 */
	public static UmlUseCaseDiagram create(UmlItem parent, String s) throws PlugoutException {
		return (UmlUseCaseDiagram)parent.create_(anItemKind.anUseCaseDiagram, s);
	}

	private UmlUseCaseDiagramDefinition _def;

	/**
	 * the constructor, do not call it yourself !!!!!!!!!!
	 */
	protected UmlUseCaseDiagram(long id, String n) {
		super(id, n);
		_def = null;
	}

	/**
	 * return the semantic part of the diagram not present in the model
	 */
	public UmlUseCaseDiagramDefinition definition() {
		if(_def == null) {
			UmlCom.send_cmd(identifier_(), OnInstanceCmd.sideCmd);
			(_def = new UmlUseCaseDiagramDefinition()).read_();
		}
		return _def;
	}

	/**
	 * returns the kind of the item
	 */
	@Override
	public anItemKind kind() {
		return anItemKind.anUseCaseDiagram;
	}

	/**
	 * to unload the object to free memory, it will be reloaded automatically if
	 * needed. args unused
	 */
	@Override
	public void unload(boolean rec, boolean del) {
		_def = null;
		super.unload(rec, del);
	}
}
