package org.bouml.plugout.apibase.umlitem;

import org.bouml.plugout.apibase.PlugoutException;
import org.bouml.plugout.apibase.com.OnInstanceCmd;
import org.bouml.plugout.apibase.com.UmlCom;
import org.bouml.plugout.apibase.type.anItemKind;

public class UmlValueSpecificationAction extends UmlActivityAction {
	/**
	 * returns a new value specification action named 's' created under 'parent'
	 * In case it cannot be created (the name is already used or invalid,
	 * 'parent' cannot contain it etc ...) return 0 in C++ and produce a
	 * PlugoutException in Java
	 */
	public static UmlValueSpecificationAction create(UmlItem parent, String s) throws PlugoutException {
		return (UmlValueSpecificationAction)parent.create_(anItemKind.aValueSpecificationAction, s);
	}

	private String _cpp_value;

	private String _java_value;

	private String _value;

	/**
	 * the constructor, do not call it yourself !!!!!!!!!!
	 */
	protected UmlValueSpecificationAction(long id, String s) {
		super(id, s);
	}

	/**
	 * return the value in C++
	 */
	public String cppValue() {
		read_if_needed_();
		return _cpp_value;
	}

	/**
	 * return the value in Java
	 */
	public String javaValue() {
		read_if_needed_();
		return _java_value;
	}

	/**
	 * returns the kind of the item
	 */
	@Override
	public anItemKind kind() {
		return anItemKind.aValueSpecificationAction;
	}

	@Override
	protected void read_cpp_() {
		super.read_cpp_();
		_cpp_value = UmlCom.read_string();
	}

	@Override
	protected void read_java_() {
		super.read_java_();
		_java_value = UmlCom.read_string();
	}

	@Override
	protected void read_uml_() {
		super.read_uml_();
		_value = UmlCom.read_string();
	}

	/**
	 * set the value in C++
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_CppValue(String v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setCppActivityCmd, v);
		UmlCom.check();
		_cpp_value = v;
	}

	/**
	 * set the value in Java
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_JavaValue(String v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setJavaActivityCmd, v);
		UmlCom.check();
		_java_value = v;
	}

	/**
	 * set the value
	 * On error return FALSE in C++, produce a PlugoutException in Java
	 */
	public void set_Value(String v) throws PlugoutException {
		UmlCom.send_cmd(identifier_(), OnInstanceCmd.setUmlActivityCmd, v);
		UmlCom.check();
		_value = v;
	}

	/**
	 * to unload the object to free memory, it will be reloaded automatically if
	 * needed. Recursively done for the sub items if 'rec' is TRUE.
	 * if 'del' is true the sub items are deleted in C++, and removed from the
	 * internal dictionnary in C++ and Java (to allow it to be garbaged), you
	 * will have to call Children() to re-access to them
	 */
	@Override
	public void unload(boolean rec, boolean del) {
		_value = null;
		_cpp_value = null;
		_java_value = null;
		super.unload(rec, del);
	}

	/**
	 * return the value
	 */
	public String value() {
		read_if_needed_();
		return _value;
	}
}
