package org.bouml.plugout.sort.controller;

import org.bouml.plugout.sort.model.SortItemKind;
import org.bouml.plugout.sort.model.SortManager;
import org.bouml.plugout.sort.view.ClassComponent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.bouml.plugout.sort.controller.SortItemController.ISortItemControllerListener;
/**
 * Controller of the class component. It fill all table of component.
 */
public class ClassController {
	private final ClassComponent component;

	private final ISortItemControllerListener listener;

	private final SortItemController memberRelationController;

	private final SortItemController memberTypeController;

	private final SortItemController memberVisibilityController;

	private final SortItemController menberAttributeController;

	public ClassController(ClassComponent classComponent, ISortItemControllerListener listener) {
		component = classComponent;
		this.listener = listener;
		memberTypeController = new SortItemController(classComponent.getMemberTypeComponent(), listener, SortItemKind.Class_Relation, SortItemKind.Class_Attribute, SortItemKind.Class_Actor, SortItemKind.Class_Class, SortItemKind.Class_ExtraMember, SortItemKind.Class_Stereotype, SortItemKind.Class_Operation);
		memberVisibilityController = new SortItemController(classComponent.getMemberVisibilityComponent(), listener, SortItemKind.Class_VisibilityPublic, SortItemKind.Class_VisibilityProtected, SortItemKind.Class_VisibilityPrivate, SortItemKind.Class_VisibilityPackage, SortItemKind.Class_VisibilityDefault);
		memberRelationController = new SortItemController(classComponent.getMemberRelationComponent(), listener, SortItemKind.Class_Dependency, SortItemKind.Class_Generalisation, SortItemKind.Class_Realization);
		menberAttributeController = new SortItemController(classComponent.getMemberAttributeComponent(), listener, SortItemKind.Class_BasicAttribute, SortItemKind.Class_Association, SortItemKind.Class_DirectionalAssociation, SortItemKind.Class_Aggregation, SortItemKind.Class_AggregationByValue, SortItemKind.Class_DirectionalAggregation, SortItemKind.Class_DirectionalAggregationByValue);
	}

	public void defineListener() {
		memberTypeController.defineListener();
		memberVisibilityController.defineListener();
		memberRelationController.defineListener();
		menberAttributeController.defineListener();
		final SelectionListener selectionListener = new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				manageCheckBoxSelection();
			}
		};
		component.getButtonClassConstructorOnFirst().addSelectionListener(selectionListener);
		component.getButtonClassStaticMemberOnFirst().addSelectionListener(selectionListener);
	}

	public void loadData() {
		memberTypeController.loadData();
		memberVisibilityController.loadData();
		memberRelationController.loadData();
		menberAttributeController.loadData();
		final SortManager sortManager = SortManager.getSingleton();
		component.getButtonClassStaticMemberOnFirst().setSelection("true".equals(sortManager.getValueOfKey(SortItemKind.Class_Constructor)) ? true : false);
		component.getButtonClassConstructorOnFirst().setSelection("true".equals(sortManager.getValueOfKey(SortItemKind.Class_StaticMember)) ? true : false);
	}

	private final void manageCheckBoxSelection() {
		final SortManager sortManager = SortManager.getSingleton();
		sortManager.setValueOfKey(SortItemKind.Class_StaticMember, component.getButtonClassStaticMemberOnFirst().getSelection() ? "true" : "false");
		sortManager.setValueOfKey(SortItemKind.Class_Constructor, component.getButtonClassConstructorOnFirst().getSelection() ? "true" : "false");
		listener.configurationChangedEvent(sortManager.isPropertiesChanged());
	}

}
