package org.bouml.plugout.sort.controller;

import java.util.ArrayList;
import java.util.List;
import org.bouml.plugout.apibase.com.UmlCom;
import org.bouml.plugout.apibase.type.anItemKind;
import org.bouml.plugout.sort.model.SortManager;
import org.bouml.plugout.sort.util.QuestionDialog;
import org.bouml.plugout.sort.view.SortConfigurationDialog;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.ShellAdapter;
import org.eclipse.swt.events.ShellEvent;
import org.bouml.plugout.sort.controller.SortItemController.ISortItemControllerListener;
/**
 * Controller of the dialog box of configuration of plug-out sort.
 */
public final class SortConfigurationController {
	private final ClassController classController;

	private boolean closedByButtons;

	private final SortConfigurationDialog dialog;

	private final List<SortItemController> sortItemControllers;

	private boolean sortNecessary;

	public SortConfigurationController() {
		closedByButtons = false;
		sortNecessary = false;
		dialog = new SortConfigurationDialog();
		final ISortItemControllerListener listener = new ISortItemControllerListener() {
			public void configurationChangedEvent(boolean modified) {
				dialog.getButtonSave().setEnabled(modified);
			}
		};
		classController = new ClassController(dialog.getClassComponent(), listener);
		sortItemControllers = new ArrayList<SortItemController>(11);
		sortItemControllers.add(new SortItemController(dialog.getActivityComponent(), listener, "Activity"));
		sortItemControllers.add(new SortItemController(dialog.getActivityActionComponent(), listener, "ActivityAction"));
		sortItemControllers.add(new SortItemController(dialog.getClassViewComponent(), listener, "ClassView"));
		sortItemControllers.add(new SortItemController(dialog.getComponentViewComponent(), listener, "ComponentView"));
		sortItemControllers.add(new SortItemController(dialog.getDeploymentViewComponent(), listener, "DeploymentView"));
		sortItemControllers.add(new SortItemController(dialog.getExpansionRegionComponent(), listener, "ExpansionRegion"));
		sortItemControllers.add(new SortItemController(dialog.getInterruptibleActivityRegionComponent(), listener, "InterruptibleActivityRegion"));
		sortItemControllers.add(new SortItemController(dialog.getPackageComponent(), listener, "Package"));
		sortItemControllers.add(new SortItemController(dialog.getStateMachineComponent(), listener, "StateMachine"));
		sortItemControllers.add(new SortItemController(dialog.getUseCaseComponent(), listener, "UseCase"));
		sortItemControllers.add(new SortItemController(dialog.getUseCaseViewComponent(), listener, "UseCaseView"));
	}

	public boolean isSortNecessary() {
		return sortNecessary;
	}

	public void showDialog(anItemKind itemKind) {
		defineListener();
		loadData();
		// Searching the appropriate tab to show.
		final CTabFolder tabFolder = dialog.getTabFolder();
		final int itemKindValue = itemKind.value();
		int indexGoodTab = -1;
		Object data;
		for(int index = tabFolder.getItemCount() - 1 ; index >= 0 ; index--) {
			data = tabFolder.getItem(index).getData();
			if(data != null && ((anItemKind)data).value() == itemKindValue) {
				indexGoodTab = index;
				index = -1;
			}
		}
		if(indexGoodTab != -1) {
			tabFolder.setSelection(indexGoodTab);
		}
		dialog.open();
	}

	private final void close() {
		if(SortManager.getSingleton().isPropertiesChanged()) {
			final QuestionDialog questionDialog = new QuestionDialog(dialog.getShell(), "The configuration has changed.\nWould you save ?");
			if(questionDialog.open()) {
				UmlCom.trace("The configuration has been saved.");
			} else {
				UmlCom.trace("The configuration was not saved.");
			}
		} else {
			UmlCom.trace("The configuration has not changed.");
		}
	}

	private final void defineListener() {
		classController.defineListener();
		for(final SortItemController sortItemController : sortItemControllers) {
			sortItemController.defineListener();
		}
		dialog.getShell().addShellListener(new ShellAdapter() {
			@Override
			public void shellClosed(ShellEvent event) {
				manageShellClosed();
			}
		});
		dialog.getButtonSort().addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent event) {
				manageButtonSort();
			}
		});
		dialog.getButtonSave().addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent event) {
				manageButtonSave();
			}
		});
		dialog.getButtonCancel().addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent event) {
				manageButtonCancel();
			}
		});
	}

	private final void loadData() {
		classController.loadData();
		for(final SortItemController sortItemController : sortItemControllers) {
			sortItemController.loadData();
		}
	}

	private final void manageButtonCancel() {
		closedByButtons = true;
		close();
		dialog.getShell().close();
	}

	private final void manageButtonSave() {
		closedByButtons = true;
		if(SortManager.getSingleton().isPropertiesChanged()) {
			SortManager.getSingleton().saveProperties();
			UmlCom.trace("The configuration has been saved.");
		} else {
			UmlCom.trace("The configuration has not changed.");
		}
		dialog.getShell().close();
	}

	private final void manageButtonSort() {
		sortNecessary = true;
		manageButtonSave();
	}

	private final void manageShellClosed() {
		if(!closedByButtons) {
			close();
		}
	}

}
