package org.bouml.plugout.sort.util;

import org.bouml.plugout.sort.resource.ISortConstant;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

/**
 * Class representing a dialog box asking a question.
 */
public final class QuestionDialog {
	private boolean answer;

	private final Shell shell;

	public QuestionDialog(Shell parent, String question) {
		answer = false;
		shell = new Shell(parent, SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
		shell.setText("Sort Configuration " + ISortConstant.VERSION);
		shell.setLayout(new GridLayout(1, true));
		shell.setLayoutData(new GridData(GridData.FILL, GridData.FILL, true, true));
		shell.setImage(ImageUtility.loadImage(ISortConstant.IMAGE_DIRECTORY + "bouml.png"));
		final Label label = new Label(shell, SWT.LEFT);
		label.setLayoutData(new GridData(GridData.FILL, GridData.FILL, true, true));
		label.setText(question);
		final Composite compositeButtons = new Composite(shell, SWT.NONE);
		compositeButtons.setLayout(new GridLayout(2, true));
		compositeButtons.setLayoutData(new GridData(GridData.FILL, GridData.FILL, true, true));
		final Button buttonYes = new Button(compositeButtons, SWT.NONE);
		buttonYes.setLayoutData(new GridData(GridData.FILL, GridData.FILL, true, true));
		buttonYes.setText("&Yes");
		final Button buttonNo = new Button(compositeButtons, SWT.NONE);
		buttonNo.setLayoutData(new GridData(GridData.FILL, GridData.FILL, true, true));
		buttonNo.setText("&No");
		final SelectionListener listener = new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				if(e.getSource() == buttonYes) {
					answer = true;
				} else {
					answer = false;
				}
				shell.close();
			}
		};
		buttonYes.addSelectionListener(listener);
		buttonNo.addSelectionListener(listener);
	}

	public boolean open() {
		shell.pack();
		final Display display = Display.getCurrent();
		final Rectangle clientArea = display.getClientArea();
		final Point shellSize = shell.getSize();
		shell.setLocation(clientArea.width / 2 - shellSize.x / 2, clientArea.height / 2 - shellSize.y / 2);
		shell.open();
		while(!shell.isDisposed()) {
			if(!display.readAndDispatch()) {
				display.sleep();
			}
		}
		return answer;
	}

}
