package org.bouml.plugout.sort.view;

import org.bouml.plugout.apibase.type.anItemKind;
import org.bouml.plugout.sort.model.SortManager;
import org.bouml.plugout.sort.resource.ISortConstant;
import org.bouml.plugout.sort.util.ImageUtility;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Link;
import org.eclipse.swt.widgets.Shell;

/**
 * Class representing the view part of the dialog box of configuration of the sort plug-out.
 */
public final class SortConfigurationDialog {
	private final SortItemComponent activityActionComponent;

	private final SortItemComponent activityComponent;

	private Button buttonCancel;

	private Button buttonSave;

	private Button buttonSort;

	private final ClassComponent classComponent;

	private final SortItemComponent classViewComponent;

	private final SortItemComponent componentViewComponent;

	private final SortItemComponent deploymentViewComponent;

	private final Display display;

	private final SortItemComponent expansionRegionComponent;

	private final SortItemComponent interruptibleActivityRegionComponent;

	private final SortItemComponent packageComponent;

	private final Shell shell;

	private final SortItemComponent stateMachineComponent;

	private CTabFolder tabFolder;

	private final SortItemComponent useCaseComponent;

	private final SortItemComponent useCaseViewComponent;

	public SortConfigurationDialog() {
		display = new Display();
		shell = new Shell(display, SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
		shell.setText("Sort Configuration " + ISortConstant.VERSION);
		shell.setLayout(new GridLayout(1, true));
		shell.setLayoutData(new GridData(GridData.FILL, GridData.FILL, true, true));
		shell.setImage(ImageUtility.loadImage(ISortConstant.IMAGE_DIRECTORY + "bouml.png"));
		activityComponent = new SortItemComponent();
		activityActionComponent = new SortItemComponent();
		classComponent = new ClassComponent();
		classViewComponent = new SortItemComponent();
		componentViewComponent = new SortItemComponent();
		deploymentViewComponent = new SortItemComponent();
		expansionRegionComponent = new SortItemComponent();
		interruptibleActivityRegionComponent = new SortItemComponent();
		packageComponent = new SortItemComponent();
		stateMachineComponent = new SortItemComponent();
		useCaseComponent = new SortItemComponent();
		useCaseViewComponent = new SortItemComponent();
		createBody();
	}

	public SortItemComponent getActivityActionComponent() {
		return activityActionComponent;
	}

	public SortItemComponent getActivityComponent() {
		return activityComponent;
	}

	public Button getButtonCancel() {
		return buttonCancel;
	}

	public Button getButtonSave() {
		return buttonSave;
	}

	public Button getButtonSort() {
		return buttonSort;
	}

	public ClassComponent getClassComponent() {
		return classComponent;
	}

	public SortItemComponent getClassViewComponent() {
		return classViewComponent;
	}

	public SortItemComponent getComponentViewComponent() {
		return componentViewComponent;
	}

	public SortItemComponent getDeploymentViewComponent() {
		return deploymentViewComponent;
	}

	public SortItemComponent getExpansionRegionComponent() {
		return expansionRegionComponent;
	}

	public SortItemComponent getInterruptibleActivityRegionComponent() {
		return interruptibleActivityRegionComponent;
	}

	public SortItemComponent getPackageComponent() {
		return packageComponent;
	}

	public Shell getShell() {
		return shell;
	}

	public SortItemComponent getStateMachineComponent() {
		return stateMachineComponent;
	}

	public CTabFolder getTabFolder() {
		return tabFolder;
	}

	public SortItemComponent getUseCaseComponent() {
		return useCaseComponent;
	}

	public SortItemComponent getUseCaseViewComponent() {
		return useCaseViewComponent;
	}

	public void open() {
		shell.pack();
		final Rectangle clientArea = display.getClientArea();
		final Point shellSize = shell.getSize();
		shell.setLocation(clientArea.width / 2 - shellSize.x / 2, clientArea.height / 2 - shellSize.y / 2);
		shell.open();
		while(!shell.isDisposed()) {
			if(!display.readAndDispatch()) {
				display.sleep();
			}
		}
	}

	private final void buildTabItem(String label, String imageName, anItemKind itemKind, SortItemComponent sortItemComponent) {
		final Composite composite = new Composite(tabFolder, SWT.NONE);
		composite.setLayout(new GridLayout(1, true));
		composite.setLayoutData(new GridData(GridData.FILL, GridData.FILL, true, true));
		final CTabItem tabItem = new CTabItem(tabFolder, SWT.NONE);
		tabItem.setText(label);
		tabItem.setControl(composite);
		tabItem.setImage(ImageUtility.loadImage(ISortConstant.IMAGE_DIRECTORY + imageName));
		tabItem.setData(itemKind);
		tabItem.setToolTipText(label);
		sortItemComponent.createBody(composite);
	}

	private final void createBody() {
		tabFolder = new CTabFolder(shell, SWT.BORDER);
		tabFolder.setLayout(new GridLayout(1, true));
		GridData tmpGridData = new GridData(GridData.FILL, GridData.FILL, true, true);
		tmpGridData.minimumWidth = 350;
		tmpGridData.widthHint = tmpGridData.minimumWidth;
		tabFolder.setLayoutData(tmpGridData);
		tabFolder.setSimple(false);
		tabFolder.setMRUVisible(true);
		tabFolder.setSelectionBackground(tabFolder.getBackground());
		buildTabItem("Activity", "activity.png", anItemKind.anActivity, activityComponent);
		buildTabItem("Activity Action", "activity_action.png", anItemKind.anOpaqueAction, activityActionComponent);
		classComponent.createBody(tabFolder);
		buildTabItem("Class View", "class_view.png", anItemKind.aClassView, classViewComponent);
		buildTabItem("Component View", "component_view.png", anItemKind.aComponentView, componentViewComponent);
		buildTabItem("Deployment View", "deployment_view.png", anItemKind.aDeploymentView, deploymentViewComponent);
		buildTabItem("Expansion Region", "expansion_region.png", anItemKind.anExpansionRegion, expansionRegionComponent);
		buildTabItem("Interruptible Activity Region", "interruptible_activity_region.png", anItemKind.anInterruptibleActivityRegion, interruptibleActivityRegionComponent);
		buildTabItem("Package/Profile", "package.png", anItemKind.aPackage, packageComponent);
		buildTabItem("State/State Machine/Submachine", "state_machine.png", anItemKind.aState, stateMachineComponent);
		buildTabItem("Use Case", "use_case.png", anItemKind.anUseCase, useCaseComponent);
		buildTabItem("Use Case View", "use_case_view.png", anItemKind.anUseCaseView, useCaseViewComponent);
		/*********************************************/
		final Composite aboutComposite = new Composite(tabFolder, SWT.NONE);
		aboutComposite.setLayout(new GridLayout(2, false));
		aboutComposite.setLayoutData(new GridData(GridData.BEGINNING, GridData.BEGINNING, false, false));
		final CTabItem aboutTabItem = new CTabItem(tabFolder, SWT.NONE);
		aboutTabItem.setText("About");
		aboutTabItem.setControl(aboutComposite);
		aboutTabItem.setImage(ImageUtility.loadImage(ISortConstant.IMAGE_DIRECTORY + "bouml.png"));
		aboutTabItem.setToolTipText(aboutTabItem.getText());
		final Label labelAboutImage = new Label(aboutComposite, SWT.BORDER);
		labelAboutImage.setLayoutData(new GridData(GridData.BEGINNING, GridData.BEGINNING, false, false));
		labelAboutImage.setImage(ImageUtility.loadImage(ISortConstant.LOPEZ_SEBASTIEN_IMAGE));
		final Composite aboutSubComposite = new Composite(aboutComposite, SWT.NONE);
		aboutSubComposite.setLayout(new GridLayout(1, true));
		aboutSubComposite.setLayoutData(new GridData(GridData.BEGINNING, GridData.BEGINNING, false, false));
		final Label labelAboutText = new Label(aboutSubComposite, SWT.LEFT);
		labelAboutText.setLayoutData(new GridData(GridData.BEGINNING, GridData.CENTER, true, false));
		final StringBuilder aboutString = new StringBuilder();
		aboutString.append(ISortConstant.PLUGOUT_SORT_NAME).append(" release ").append(ISortConstant.VERSION).append('\n').append("LOPEZ Sébastien\n");
		labelAboutText.setText(aboutString.toString());
		new Link(aboutSubComposite, SWT.NONE).setText("<A>lopseb14@hotmail.com</A>\n<A>http://bouml.free.fr/contrib.html</A>");
		/*********************************************/
		final Composite compositeButtons = new Composite(shell, SWT.NONE);
		compositeButtons.setLayout(new GridLayout(3, true));
		compositeButtons.setLayoutData(new GridData(GridData.FILL, GridData.FILL, true, true));
		buttonSort = new Button(compositeButtons, SWT.NONE);
		tmpGridData = new GridData(GridData.CENTER, GridData.FILL, true, true);
		tmpGridData.minimumWidth = ISortConstant.BUTTON_WIDTH;
		tmpGridData.widthHint = tmpGridData.minimumWidth;
		buttonSort.setLayoutData(tmpGridData);
		buttonSort.setText("&Sort");
		buttonSort.setToolTipText("Save the configuration and launch the plugout sort.");
		buttonSave = new Button(compositeButtons, SWT.NONE);
		tmpGridData = new GridData(GridData.CENTER, GridData.FILL, true, true);
		tmpGridData.minimumWidth = ISortConstant.BUTTON_WIDTH;
		tmpGridData.widthHint = tmpGridData.minimumWidth;
		buttonSave.setLayoutData(tmpGridData);
		buttonSave.setText("&Save");
		buttonSave.setToolTipText("Save the configuration.");
		buttonSave.setEnabled(SortManager.getSingleton().isPropertiesChanged());
		buttonCancel = new Button(compositeButtons, SWT.NONE);
		tmpGridData = new GridData(GridData.CENTER, GridData.FILL, true, true);
		tmpGridData.minimumWidth = ISortConstant.BUTTON_WIDTH;
		tmpGridData.widthHint = tmpGridData.minimumWidth;
		buttonCancel.setLayoutData(tmpGridData);
		buttonCancel.setText("&Cancel");
		buttonCancel.setToolTipText("Don't save the configuration and close the dialog box.");
	}

}
