package org.bouml.plugout.sort;

import org.bouml.plugout.apibase.AbstractPlugout;
import org.bouml.plugout.apibase.com.UmlCom;
import org.bouml.plugout.apibase.type.anItemKind;
import org.bouml.plugout.apibase.umlitem.UmlItem;
import org.bouml.plugout.apibase.umlitem.UmlItemFactory;
import org.bouml.plugout.sort.controller.SortConfigurationController;
import org.bouml.plugout.sort.model.ISortItem;
import org.bouml.plugout.sort.model.SortItemComparator;
import org.bouml.plugout.sort.model.SortManager;
import org.bouml.plugout.sort.model.UmlActivity;
import org.bouml.plugout.sort.model.UmlActivityDiagram;
import org.bouml.plugout.sort.model.UmlActivityObject;
import org.bouml.plugout.sort.model.UmlActivityParameter;
import org.bouml.plugout.sort.model.UmlActivityPartition;
import org.bouml.plugout.sort.model.UmlActivityPin;
import org.bouml.plugout.sort.model.UmlArtifact;
import org.bouml.plugout.sort.model.UmlAttribute;
import org.bouml.plugout.sort.model.UmlClass;
import org.bouml.plugout.sort.model.UmlClassDiagram;
import org.bouml.plugout.sort.model.UmlClassInstance;
import org.bouml.plugout.sort.model.UmlClassView;
import org.bouml.plugout.sort.model.UmlCollaborationDiagram;
import org.bouml.plugout.sort.model.UmlComponent;
import org.bouml.plugout.sort.model.UmlComponentDiagram;
import org.bouml.plugout.sort.model.UmlComponentView;
import org.bouml.plugout.sort.model.UmlDeploymentDiagram;
import org.bouml.plugout.sort.model.UmlDeploymentView;
import org.bouml.plugout.sort.model.UmlExpansionNode;
import org.bouml.plugout.sort.model.UmlExpansionRegion;
import org.bouml.plugout.sort.model.UmlExtraClassMember;
import org.bouml.plugout.sort.model.UmlInterruptibleActivityRegion;
import org.bouml.plugout.sort.model.UmlNode;
import org.bouml.plugout.sort.model.UmlObjectDiagram;
import org.bouml.plugout.sort.model.UmlOpaqueAction;
import org.bouml.plugout.sort.model.UmlOperation;
import org.bouml.plugout.sort.model.UmlPackage;
import org.bouml.plugout.sort.model.UmlParameterSet;
import org.bouml.plugout.sort.model.UmlRegion;
import org.bouml.plugout.sort.model.UmlRelation;
import org.bouml.plugout.sort.model.UmlSequenceDiagram;
import org.bouml.plugout.sort.model.UmlState;
import org.bouml.plugout.sort.model.UmlStateDiagram;
import org.bouml.plugout.sort.model.UmlUseCase;
import org.bouml.plugout.sort.model.UmlUseCaseDiagram;
import org.bouml.plugout.sort.model.UmlUseCaseView;
import org.bouml.plugout.sort.resource.ISortConstant;

/**
 * Class representing the plug-out sort.
 */
public final class Sort extends AbstractPlugout {
	private final boolean configuration;

	public static void main(String[] args) {
		if(args.length > 0) {
			new Sort(args).launch(Integer.parseInt(args[0]));
		} else {
			System.err.println("The socket number is missing.");
		}
	}

	public Sort(String[] args) {
		super(ISortConstant.PLUGOUT_SORT_NAME, ISortConstant.VERSION);
		
		if(args.length > 1 && ISortConstant.CONFIGURATION_ARG.equals(args[1])) {
			configuration = true;
		} else {
			configuration = false;
		}
	}

	@Override
	protected void configureUmlItemFactory(UmlItemFactory factory) {
		factory.setItems(anItemKind.anActivity, UmlActivity.class);
		factory.setItems(anItemKind.anActivityDiagram, UmlActivityDiagram.class);
		factory.setItems(anItemKind.anActivityObject, UmlActivityObject.class);
		factory.setItems(anItemKind.anActivityParameter, UmlActivityParameter.class);
		factory.setItems(anItemKind.anActivityPin, UmlActivityPin.class);
		factory.setItems(anItemKind.anArtifact, UmlArtifact.class);
		factory.setItems(anItemKind.anAttribute, UmlAttribute.class);
		factory.setItems(anItemKind.aClass, UmlClass.class);
		factory.setItems(anItemKind.aClassDiagram, UmlClassDiagram.class);
		factory.setItems(anItemKind.aClassInstance, UmlClassInstance.class);
		factory.setItems(anItemKind.aClassView, UmlClassView.class);
		factory.setItems(anItemKind.aCollaborationDiagram, UmlCollaborationDiagram.class);
		factory.setItems(anItemKind.aComponent, UmlComponent.class);
		factory.setItems(anItemKind.aComponentDiagram, UmlComponentDiagram.class);
		factory.setItems(anItemKind.aComponentView, UmlComponentView.class);
		factory.setItems(anItemKind.aDeploymentDiagram, UmlDeploymentDiagram.class);
		factory.setItems(anItemKind.aDeploymentView, UmlDeploymentView.class);
		factory.setItems(anItemKind.anExpansionNode, UmlExpansionNode.class);
		factory.setItems(anItemKind.anExpansionRegion, UmlExpansionRegion.class);
		factory.setItems(anItemKind.anExtraClassMember, UmlExtraClassMember.class);
		factory.setItems(anItemKind.anInterruptibleActivityRegion, UmlInterruptibleActivityRegion.class);
		factory.setItems(anItemKind.aNode, UmlNode.class);
		factory.setItems(anItemKind.anObjectDiagram, UmlObjectDiagram.class);
		factory.setItems(anItemKind.anOpaqueAction, UmlOpaqueAction.class);
		factory.setItems(anItemKind.anOperation, UmlOperation.class);
		factory.setItems(anItemKind.aPackage, UmlPackage.class);
		factory.setItems(anItemKind.aParameterSet, UmlParameterSet.class);
		factory.setItems(anItemKind.aPartition, UmlActivityPartition.class);
		factory.setItems(anItemKind.aRegion, UmlRegion.class);
		factory.setItems(anItemKind.aRelation, UmlRelation.class);
		factory.setItems(anItemKind.aSequenceDiagram, UmlSequenceDiagram.class);
		factory.setItems(anItemKind.aState, UmlState.class);
		factory.setItems(anItemKind.aStateDiagram, UmlStateDiagram.class);
		factory.setItems(anItemKind.anUseCase, UmlUseCase.class);
		factory.setItems(anItemKind.anUseCaseDiagram, UmlUseCaseDiagram.class);
		factory.setItems(anItemKind.anUseCaseView, UmlUseCaseView.class);
	}

	@Override
	protected void managePlugout(UmlItem item) {
		SortManager.getSingleton();
		SortItemComparator.getSingleton();

		try {
			if(configuration) {
				final SortConfigurationController sortConfigurationController = new SortConfigurationController();
				sortConfigurationController.showDialog(item.kind());
				if(sortConfigurationController.isSortNecessary()) {
					sort(item);
				}
			} else {
				sort(item);
			}
		} catch(final Exception exception) {
			UmlCom.traceException(exception);
		}
	}

	private final void sort(UmlItem rootItem) {
		if(rootItem != null && rootItem instanceof ISortItem) {
			final UmlPackage umlPackage = (UmlPackage)org.bouml.plugout.apibase.umlitem.UmlPackage.getProject();
			final boolean childrenVisible = umlPackage.childrenVisible();
			umlPackage.set_childrenVisible(false);
			UmlCom.trace(" ");
			((ISortItem)rootItem).sort();
			UmlCom.trace("<hr>Number of containers sorted : " + Integer.toString(SortItemComparator.getSingleton().getNbSortingItems()) + "<br>");
			umlPackage.set_childrenVisible(childrenVisible);
		} else {
			UmlCom.traceError("Item unknow : " + rootItem.name());
		}
	}

}
