package org.bouml.plugout.sort.model;

import org.bouml.plugout.sort.resource.ISortConstant;

/**
 * Enumeration of all elements that can be sorted.
 */
public enum SortItemKind {
	ActivityAction_ParameterSet,
	ActivityAction_Pin,
	Activity_ActivityAction,
	Activity_ActivityDiagram,
	Activity_ExpansionRegion,
	Activity_InterruptibleActivityRegion,
	Activity_ObjectNode,
	Activity_Parameter,
	Activity_Partition,
	ClassView_Activity,
	ClassView_Actor,
	ClassView_Class,
	ClassView_ClassDiagram,
	ClassView_ClassInstance,
	ClassView_CommunicationDiagram,
	ClassView_ObjectDiagram,
	ClassView_SequenceDiagram,
	ClassView_State("state_machine.png", null),
	ClassView_Stereotype,
	Class_Actor,
	Class_Aggregation,
	Class_AggregationByValue,
	Class_Association,
	Class_Attribute,
	Class_BasicAttribute("attribute.png", null),
	Class_Class,
	Class_Constructor,
	Class_Dependency,
	Class_DirectionalAggregation,
	Class_DirectionalAggregationByValue,
	Class_DirectionalAssociation,
	Class_ExtraMember,
	Class_Generalisation,
	Class_Operation,
	Class_Realization,
	Class_Relation,
	Class_StaticMember,
	Class_Stereotype,
	Class_VisibilityDefault("visibility_package.png", "Default"),
	Class_VisibilityPackage(null, "Package"),
	Class_VisibilityPrivate(null, "Private"),
	Class_VisibilityProtected(null, "Protected"),
	Class_VisibilityPublic(null, "Public"),
	ComponentView_Component,
	ComponentView_ComponentDiagram,
	DeploymentView_Artifact,
	DeploymentView_DeploymentDiagram,
	DeploymentView_Node,
	ExpansionRegion_ActivityAction,
	ExpansionRegion_ExpansionNode,
	ExpansionRegion_ExpansionRegion,
	ExpansionRegion_InterruptibleActivityRegion,
	ExpansionRegion_ObjectNode,
	InterruptibleActivityRegion_ActivityAction,
	InterruptibleActivityRegion_ExpansionRegion,
	InterruptibleActivityRegion_InterruptibleActivityRegion,
	InterruptibleActivityRegion_ObjectNode,
	Package_ClassView,
	Package_ComponentView,
	Package_Dependency,
	Package_DeploymentView,
	Package_Generalisation,
	Package_Package,
	Package_Profile,
	Package_UseCaseView,
	StateMachine_Region,
	StateMachine_State("state_machine.png", null),
	StateMachine_StateDiagram,
	StateMachine_SubMachine("state_machine.png", null),
	UseCaseView_Activity,
	UseCaseView_Actor,
	UseCaseView_Class,
	UseCaseView_ClassInstance,
	UseCaseView_CommunicationDiagram,
	UseCaseView_ObjectDiagram,
	UseCaseView_SequenceDiagram,
	UseCaseView_StateMachine,
	UseCaseView_Stereotype,
	UseCaseView_SubUseCaseView("use_case_view.png", null),
	UseCaseView_UseCase,
	UseCaseView_UseCaseDiagram,
	UseCase_Activity,
	UseCase_Actor,
	UseCase_Class,
	UseCase_ClassInstance,
	UseCase_CommunicationDiagram,
	UseCase_ObjectDiagram,
	UseCase_SequenceDiagram,
	UseCase_StateMachine,
	UseCase_Stereotype,
	UseCase_UseCase,
	UseCase_UseCaseDiagram,;
	private final String imagePath;

	private final String label;

	private SortItemKind() {
		imagePath = buildImagePath();
		label = buildLabel();
	}

	private SortItemKind(String imageName, String label) {
		imagePath = imageName == null ? buildImagePath() : ISortConstant.IMAGE_DIRECTORY + imageName;
		this.label = label == null ? buildLabel() : label;
	}

	public String getImagePath() {
		return imagePath;
	}

	public String getLabel() {
		return label;
	}

	public int getPosition() {
		return SortManager.getSingleton().getOrderWeight(this);
	}

	public String getPrefix() {
		return name().split(ISortConstant.ITEM_PREFIX_SUFFIX_SEPARATOR)[0];
	}

	public void setPosition(int position) {
		SortManager.getSingleton().setValueOfKey(this, Integer.toString(position));
	}

	/**
	 * Method to construct the corresponding path of image to the item.
	 * @return The image path in a String.
	 */
	private final String buildImagePath() {
		final String item = name().split(ISortConstant.ITEM_PREFIX_SUFFIX_SEPARATOR)[1];
		final StringBuffer tmpString = new StringBuffer();
		char tmpChar;
		tmpString.append(ISortConstant.IMAGE_DIRECTORY).append(Character.toLowerCase(item.charAt(0)));
		for(int index = 1 ; index < item.length() ; index++) {
			tmpChar = item.charAt(index);
			if(Character.isUpperCase(tmpChar)) {
				tmpString.append(ISortConstant.ITEM_PREFIX_SUFFIX_SEPARATOR).append(Character.toLowerCase(tmpChar));
			} else {
				tmpString.append(tmpChar);
			}
		}
		return tmpString.append(ISortConstant.ITEM_IMAGE_EXTENSION).toString();
	}

	/**
	 * Method to construct the corresponding label to the item.
	 * @return The label in a String.
	 */
	private final String buildLabel() {
		final String item = name().split(ISortConstant.ITEM_PREFIX_SUFFIX_SEPARATOR)[1];
		final StringBuffer tmpString = new StringBuffer();
		char tmpChar;
		tmpString.append(item.charAt(0));
		for(int index = 1 ; index < item.length() ; index++) {
			tmpChar = item.charAt(index);
			if(Character.isUpperCase(tmpChar)) {
				tmpString.append(ISortConstant.SPACE).append(tmpChar);
			} else {
				tmpString.append(tmpChar);
			}
		}
		return tmpString.toString();
	}

}
