package org.bouml.plugout.sort.model;

import org.bouml.plugout.apibase.PlugoutException;
import org.bouml.plugout.apibase.type.aRelationKind;
import org.bouml.plugout.apibase.type.anItemKind;
import org.bouml.plugout.apibase.umlitem.UmlItem;
import org.bouml.plugout.sort.resource.ISortConstant;

public class UmlRelation extends org.bouml.plugout.apibase.umlitem.UmlRelation implements ISortItem {
	public UmlRelation(long id, String n) {
		super(id, n);
	}

	/**
	 * Get the path of the include.
	 * @return The path of the include.
	 */
	public String getCppHDir() {
		final UmlClass umlClass = (UmlClass)roleType();
		if(umlClass != null) {
			final UmlItem umlItem = umlClass.parent();
			if(umlItem != null) {
				final String cppNamespace = ((UmlPackage)umlItem.parent()).cppHDir();
				return cppNamespace == null ? "" : cppNamespace;
			}
		}
		return "";
	}

	/**
	 * Get the value of the import.
	 * @return The value of the import.
	 */
	public String getJavaImport() {
		final UmlClass umlClass = (UmlClass)roleType();
		if(umlClass != null) {
			UmlItem umlItem = umlClass.parent();
			String javaImport = '.' + umlClass.name();
			while(umlItem != null && !(umlItem instanceof UmlPackage)) {
				if(umlItem instanceof UmlClass) {
					javaImport = '.' + umlItem.name() + javaImport;
				}
				umlItem = umlItem.parent();
			}
			if(umlItem != null) {
				final String javaPackage = ((UmlPackage)umlItem).javaPackage();
				return javaPackage == null ? "" : javaPackage + javaImport;
			}
		}
		return "";
	}

	public int getOrderWeight() {
		final int parentKindValue = parent().kind().value();
		if(parentKindValue == anItemKind._aClass) {
			return SortManager.getSingleton().getClassMemberOrderWeight(this);
		} else if(parentKindValue == anItemKind._aPackage) {
			return SortManager.getSingleton().getOrderWeight(relationKind().value() == aRelationKind._aGeneralisation ? SortItemKind.Package_Generalisation : SortItemKind.Package_Dependency);
		} else {
			throw new PlugoutException(ISortConstant.MSG_PARENT_UNKNOW);
		}
	}

	public void sort() {
		// Do nothing.
	}

}
