package org.bouml.plugout.sort.view;

import org.bouml.plugout.apibase.type.anItemKind;
import org.bouml.plugout.sort.resource.ISortConstant;
import org.bouml.plugout.sort.util.ImageUtility;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;

/**
 * Class representing the component for the configuration of a class.
 */
public class ClassComponent {
	private Button buttonClassConstructorOnFirst;

	private Button buttonClassStaticMemberOnFirst;

	private final SortItemComponent memberAttributeComponent;

	private final SortItemComponent memberRelationComponent;

	private final SortItemComponent memberTypeComponent;

	private final SortItemComponent memberVisibilityComponent;

	public ClassComponent() {
		memberTypeComponent = new SortItemComponent();
		memberVisibilityComponent = new SortItemComponent();
		memberRelationComponent = new SortItemComponent();
		memberAttributeComponent = new SortItemComponent();
	}

	public void createBody(CTabFolder parent) {
		final CTabItem tabItemClass = new CTabItem(parent, SWT.NONE);
		tabItemClass.setText("Actor/Class/Stereotype");
		tabItemClass.setToolTipText("Actor/Class/Stereotype");
		tabItemClass.setImage(ImageUtility.loadImage(ISortConstant.IMAGE_DIRECTORY + "class.png"));
		tabItemClass.setData(anItemKind.aClass);
		final CTabFolder tabFolder = new CTabFolder(parent, SWT.BORDER);
		tabFolder.setLayout(new GridLayout(1, true));
		tabFolder.setLayoutData(new GridData(GridData.FILL, GridData.FILL, true, true));
		tabFolder.setSimple(false);
		tabFolder.setMRUVisible(true);
		tabFolder.setSelectionBackground(tabFolder.getBackground());
		tabItemClass.setControl(tabFolder);
		/***************************************************************************************/
		buildTabItem(tabFolder, "Attribute", memberAttributeComponent);
		buildTabItem(tabFolder, "Relation", memberRelationComponent);
		/***************************************************************************************/
		final Composite composite = new Composite(tabFolder, SWT.NONE);
		composite.setLayout(new GridLayout(1, true));
		composite.setLayoutData(new GridData(GridData.FILL, GridData.BEGINNING, true, false));
		final CTabItem tabItem = new CTabItem(tabFolder, SWT.NONE);
		tabItem.setText("Special");
		tabItem.setControl(composite);
		tabItem.setToolTipText(tabItem.getText());
		buttonClassConstructorOnFirst = new Button(composite, SWT.CHECK);
		buttonClassConstructorOnFirst.setLayoutData(new GridData(GridData.FILL, GridData.BEGINNING, true, false));
		buttonClassConstructorOnFirst.setText("Constructor on first.");
		buttonClassConstructorOnFirst.setToolTipText("Construtor may be after static operation.");
		buttonClassStaticMemberOnFirst = new Button(composite, SWT.CHECK);
		buttonClassStaticMemberOnFirst.setLayoutData(new GridData(GridData.FILL, GridData.BEGINNING, true, false));
		buttonClassStaticMemberOnFirst.setText("Static member on first.");
		buttonClassStaticMemberOnFirst.setToolTipText("Available for attribute and operation.
		/***************************************************************************************/
		buildTabItem(tabFolder, "Type", memberTypeComponent);
		buildTabItem(tabFolder, "Visibility", memberVisibilityComponent);
		/***************************************************************************************/
		tabFolder.setSelection(0);
	}

	public Button getButtonClassConstructorOnFirst() {
		return buttonClassConstructorOnFirst;
	}

	public Button getButtonClassStaticMemberOnFirst() {
		return buttonClassStaticMemberOnFirst;
	}

	public SortItemComponent getMemberAttributeComponent() {
		return memberAttributeComponent;
	}

	public SortItemComponent getMemberRelationComponent() {
		return memberRelationComponent;
	}

	public SortItemComponent getMemberTypeComponent() {
		return memberTypeComponent;
	}

	public SortItemComponent getMemberVisibilityComponent() {
		return memberVisibilityComponent;
	}

	private final void buildTabItem(CTabFolder tabFolder, String label, SortItemComponent component) {
		final Composite composite = new Composite(tabFolder, SWT.NONE);
		composite.setLayout(new GridLayout(1, true));
		composite.setLayoutData(new GridData(GridData.FILL, GridData.FILL, true, true));
		final CTabItem tabItem = new CTabItem(tabFolder, SWT.NONE);
		tabItem.setText(label);
		tabItem.setControl(composite);
		tabItem.setToolTipText(label);
		component.createBody(composite);
	}

}
